@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: caption.c,v 1.2 2002/01/21 07:57:10 mschimek Exp $ */
+/* $Id: caption.c,v 1.14 2003/10/21 20:53:05 mschimek Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -69,7 +69,7 @@ caption_send_event(vbi_decoder *vbi, vbi_event *ev)
 #define XDS_END			15
 
 /* vbi_classify_page, program_info language */
-static char *
+static const char *
 language[8] = {
 	"Unknown",
 	"English",
@@ -81,42 +81,48 @@ language[8] = {
 	"None"
 };
 
-static char *
-map_type[8] __attribute__ ((unused)) = {
+#ifdef __GNUC__
+#define UNUSED __attribute__ ((unused))
+#else
+#define UNUSED
+#endif
+
+static const char *
+map_type[] UNUSED = {
 	"unknown", "mono", "simulated stereo", "stereo",
 	"stereo surround", "data service", "unknown", "none"
 };
 
-static char *
-sap_type[8] __attribute__ ((unused)) = {
+static const char *
+sap_type[] UNUSED = {
 	"unknown", "mono", "video descriptions", "non-program audio",
 	"special effects", "data service", "unknown", "none"
 };
 
-static char *
-cgmsa[4] __attribute__ ((unused)) = {
+static const char *
+cgmsa[] UNUSED = {
 	"copying permitted",
 	"-",
 	"one generation copy allowed",
 	"no copying permitted"
 };
 
-static char *
-scrambling[4] __attribute__ ((unused)) = {
+static const char *
+scrambling[] UNUSED = {
 	"no pseudo-sync pulse",
 	"pseudo-sync pulse on; color striping off",
 	"pseudo-sync pulse on; 2-line color striping on",
 	"pseudo-sync pulse on; 4-line color striping on"
 };
 
-static char *
-month_names[] __attribute__ ((unused)) = {
+static const char *
+month_names[] UNUSED = {
 	"0?", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
 	"Sep", "Oct", "Nov", "Dec", "13?", "14?", "15?"
 };
 
-static char *
-day_names[] __attribute__ ((unused)) = {
+static const char *
+day_names[] UNUSED = {
 	"0?", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 
@@ -124,7 +130,10 @@ static uint32_t hcrc[128];
 
 static void init_hcrc(void) __attribute__ ((constructor));
 
+
 /*
+http://www.fcc.gov/cgb/statid.html
+ *
  *  XDS has no unique station id as EBU (or is the call sign?)
  *  so we create a checksum over the station name.
  */
@@ -180,7 +189,7 @@ flush_prog_info(vbi_decoder *vbi, vbi_program_info *pi, vbi_event *e)
 }
 
 static inline void
-xds_decoder(vbi_decoder *vbi, int class, int type,
+xds_decoder(vbi_decoder *vbi, int _class, int type,
 	    uint8_t *buffer, int length)
 {
 	vbi_network *n = &vbi->network.ev.network;
@@ -188,14 +197,14 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 	int neq, i;
 	vbi_event e;
 
-	assert(length > 0 && length < 32);
+	assert(length > 0 && length <= 32);
 
 // XXX we have no indication how long the program info applies.
 //     It will be canceled on channel switch, but who knows
 //     what the station transmits when the next program starts.
 //     (Nothing, possibly.) A timeout seems necessary. 
 
-	switch (class) {
+	switch (_class) {
 	case XDS_CURRENT: /* 0 */
 	case XDS_FUTURE: /* 1 */
 		XDS_DEBUG(printf((class == XDS_CURRENT) ? "Current " : "Next "));
@@ -203,7 +212,7 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 		if (!(vbi->event_mask & (VBI_EVENT_ASPECT | VBI_EVENT_PROG_INFO)))
 			return;
 
-		pi = &vbi->prog_info[class];
+		pi = &vbi->prog_info[_class];
 		neq = 0;
 
 		switch (type) {
@@ -306,16 +315,16 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 			neq = xds_strfu(pi->title, buffer, length);
 
 			if (!neq) { /* no title change */
-				if (!(vbi->cc.info_cycle[class] & (1 << 3)))
+				if (!(vbi->cc.info_cycle[_class] & (1 << 3)))
 					break; /* already reported */
 
-				if (!(vbi->cc.info_cycle[class] & (1 << 1))) {
+				if (!(vbi->cc.info_cycle[_class] & (1 << 1))) {
 					/* Second occurence without PIN */
 
 					flush_prog_info(vbi, pi, &e);
 
 					xds_strfu(pi->title, buffer, length);
-					vbi->cc.info_cycle[class] |= 1 << 3;
+					vbi->cc.info_cycle[_class] |= 1 << 3;
 				}
 			}
 
@@ -455,7 +464,7 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 			for (i = 0; i < 2; i++) {
 				int l = (buffer[i] >> 3) & 7;
 				vbi_audio_mode m = mode[i][buffer[i] & 7];
-				char *s = ((1 << l) & 0xC1) ? NULL : language[l];
+				const char *s = ((1 << l) & 0xC1) ? NULL : language[l];
 
 				if (pi->audio[i].mode != m) {
 					neq = 1; pi->audio[i].mode = m;
@@ -481,18 +490,18 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 			for (i = 0; i < length; i++) {
 				int ch = buffer[i] & 7;
 				int l = (buffer[i] >> 3) & 7;
-				char *s;
+				const char *s;
 
 				ch = (ch & 1) * 4 + (ch >> 1);
 
 				services |= 1 << ch;
 				s = ((1 << l) & 0xC1) ? NULL : language[l];
 
-				if (pi->caption_language[ch] != s) {
-					neq = 1; pi->caption_language[ch] = s;
+				if (pi->caption_language[ch] != (unsigned char *) s) {
+					neq = 1; pi->caption_language[ch] = (unsigned char *) s;
 				}
 
-				if (class == XDS_CURRENT)
+				if (_class == XDS_CURRENT)
 					vbi->cc.channel[ch].language =
 						pi->caption_language[ch];
 			}
@@ -587,13 +596,13 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 			return; /* no event */
 		}
 
-#if 0
-		printf("[type %d cycle %08x class %d neq %d]\n",
-		       type, vbi->cc.info_cycle[class], class, neq);
-#endif
+		if (0)
+			printf("[type %d cycle %08x class %d neq %d]\n",
+			       type, vbi->cc.info_cycle[_class], _class, neq);
+
 		if (neq) /* first occurence of this type with this data */
-			vbi->cc.info_cycle[class] |= 1 << type;
-		else if (vbi->cc.info_cycle[class] & (1 << type)) {
+			vbi->cc.info_cycle[_class] |= 1 << type;
+		else if (vbi->cc.info_cycle[_class] & (1 << type)) {
 			/* Second occurance of this type with same data */
 
 			e.type = VBI_EVENT_PROG_INFO;
@@ -601,7 +610,7 @@ xds_decoder(vbi_decoder *vbi, int class, int type,
 
 			caption_send_event(vbi, &e);
 
-			vbi->cc.info_cycle[class] = 0; /* all changes reported */
+			vbi->cc.info_cycle[_class] = 0; /* all changes reported */
 		}
 
 		break;
@@ -764,7 +773,7 @@ xds_separator(vbi_decoder *vbi, uint8_t *buf)
 	xds_sub_packet *sp = cc->curr_sp;
 	int c1 = vbi_parity(buf[0]);
 	int c2 = vbi_parity(buf[1]);
-	int class, type;
+	unsigned int class, type;
 
 	XDS_SEP_DEBUG(printf("XDS %02x %02x\n", buf[0], buf[1]));
 
@@ -953,7 +962,7 @@ roll_up(vbi_page *pg, int first_row, int last_row)
 }
 
 static inline void
-update(channel *ch)
+update(cc_channel *ch)
 {
 	vbi_char *acp = ch->line - ch->pg[0].text + ch->pg[1].text;
 
@@ -961,7 +970,7 @@ update(channel *ch)
 }
 
 static void
-word_break(struct caption *cc, channel *ch, int upd)
+word_break(struct caption *cc, cc_channel *ch, int upd)
 {
 	/*
 	 *  Add a leading and trailing space.
@@ -1002,7 +1011,7 @@ word_break(struct caption *cc, channel *ch, int upd)
 }
 
 static inline void
-set_cursor(channel *ch, int col, int row)
+set_cursor(cc_channel *ch, int col, int row)
 {
 	ch->col = ch->col1 = col;
 	ch->row = row;
@@ -1011,7 +1020,7 @@ set_cursor(channel *ch, int col, int row)
 }
 
 static void
-put_char(struct caption *cc, channel *ch, vbi_char c)
+put_char(struct caption *cc, cc_channel *ch, vbi_char c)
 {
 	/* c.foreground = rand() & 7; */
 	/* c.background = rand() & 7; */
@@ -1028,8 +1037,8 @@ put_char(struct caption *cc, channel *ch, vbi_char c)
 		word_break(cc, ch, 1);
 }
 
-static inline channel *
-switch_channel(struct caption *cc, channel *ch, int new_chan)
+static inline cc_channel *
+switch_channel(struct caption *cc, cc_channel *ch, int new_chan)
 {
 	word_break(cc, ch, 1); // we leave for a number of frames
 
@@ -1037,7 +1046,7 @@ switch_channel(struct caption *cc, channel *ch, int new_chan)
 }
 
 static void
-erase_memory(struct caption *cc, channel *ch, int page)
+erase_memory(struct caption *cc, cc_channel *ch, int page)
 {
 	vbi_page *pg = ch->pg + page;
 	vbi_char *acp = pg->text;
@@ -1070,7 +1079,7 @@ static inline void
 caption_command(vbi_decoder *vbi, struct caption *cc,
 	unsigned char c1, unsigned char c2, vbi_bool field2)
 {
-	channel *ch;
+	cc_channel *ch;
 	int chan, col, i;
 	int last_row;
 
@@ -1399,14 +1408,14 @@ caption_command(vbi_decoder *vbi, struct caption *cc,
 }
 
 /**
- * vbi_decode_caption:
- * @vbi: Initialized vbi decoding context.
- * @line: ITU-R line number this data originated from.
- * @buf: Two bytes.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
+ * @param line ITU-R line number this data originated from.
+ * @param buf Two bytes.
  * 
  * Decode two bytes of Closed Caption data (Caption, XDS, ITV),
  * updating the decoder state accordingly. May send events.
- **/
+ */
 void
 vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 {
@@ -1414,7 +1423,7 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 	char c1 = buf[0] & 0x7F;
 	int field2 = 1, i;
 
-	pthread_mutex_unlock(&cc->mutex);
+	pthread_mutex_lock(&cc->mutex);
 
 	switch (line) {
 	case 21:	/* NTSC */
@@ -1469,7 +1478,7 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 	)
 
 	switch (c1) {
-		channel *ch;
+		cc_channel *ch;
 		vbi_char c;
 
 	case 0x01 ... 0x0F:
@@ -1532,7 +1541,7 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 		for (i = 0; i < 2; i++) {
 			char ci = vbi_parity(buf[i]) & 0x7F; /* 127 if bad */
 
-			if (ci == 0)
+			if (ci <= 0x1F) /* 0x00 no char, 0x01 ... 0x1F invalid */
 				continue;
 
 			if (ch == cc->channel + 5) // 'T2'
@@ -1549,13 +1558,13 @@ vbi_decode_caption(vbi_decoder *vbi, int line, uint8_t *buf)
 }
 
 /**
- * vbi_caption_desync:
- * @vbi: Initialized vbi decoding context. 
+ * @internal
+ * @param vbi Initialized vbi decoding context. 
  * 
  * This function must be called after desynchronisation
  * has been detected (i. e. vbi data has been lost)
  * to reset the Closed Caption decoder.
- **/
+ */
 void
 vbi_caption_desync(vbi_decoder *vbi)
 {
@@ -1574,17 +1583,17 @@ vbi_caption_desync(vbi_decoder *vbi)
 }
 
 /**
- * vbi_caption_channel_switched:
- * @vbi: Initialized vbi decoding context.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
  * 
  * This function must be called after a channel switch,
  * to reset the Closed Caption decoder.
- **/
+ */
 void
 vbi_caption_channel_switched(vbi_decoder *vbi)
 {
 	struct caption *cc = &vbi->cc;
-	channel *ch;
+	cc_channel *ch;
 	int i;
 
 	for (i = 0; i < 9; i++) {
@@ -1639,12 +1648,12 @@ default_color_map[8] = {
 };
 
 /**
- * vbi_caption_color_level:
- * @vbi: Initialized vbi decoding context.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
  * 
  * After the client changed text brightness and saturation
  * this function adjusts the Closed Caption color palette.
- **/
+ */
 void
 vbi_caption_color_level(vbi_decoder *vbi)
 {
@@ -1660,12 +1669,12 @@ vbi_caption_color_level(vbi_decoder *vbi)
 }
 
 /**
- * vbi_caption_destroy:
- * @vbi: VBI decoding context.
+ * @internal
+ * @param vbi VBI decoding context.
  * 
- * This function is called during @vbi destruction
- * to destroy Closed Caption subset of @vbi.
- **/
+ * This function is called during @a vbi destruction
+ * to destroy Closed Caption subset of @a vbi.
+ */
 void
 vbi_caption_destroy(vbi_decoder *vbi)
 {
@@ -1673,17 +1682,17 @@ vbi_caption_destroy(vbi_decoder *vbi)
 }
 
 /**
- * vbi_caption_init:
- * @vbi: VBI decoding context.
+ * @internal
+ * @param vbi VBI decoding context.
  * 
- * This function is called during @vbi initialization
- * to initialize the Closed Caption subset of @vbi.
- **/
+ * This function is called during @a vbi initialization
+ * to initialize the Closed Caption subset of @a vbi.
+ */
 void
 vbi_caption_init(vbi_decoder *vbi)
 {
 	struct caption *cc = &vbi->cc;
-	channel *ch;
+	cc_channel *ch;
 	int i;
 
 	memset(cc, 0, sizeof(struct caption));
@@ -1725,35 +1734,34 @@ vbi_caption_init(vbi_decoder *vbi)
 }
 
 /**
- * vbi_fetch_cc_page:
- * @vbi: Initialized vbi decoding context.
- * @pg: Place to store the formatted page.
- * @pgno: Page number of the page to fetch, see #vbi_pgno.
- * @reset: TRUE resets the #vbi_page dirty fields in cache after
- *   fetching. Pass FALSE only if you plan to call this function again
+ * @param vbi Initialized vbi decoding context.
+ * @param pg Place to store the formatted page.
+ * @param pgno Page number 1 ... 8 of the page to fetch, see vbi_pgno.
+ * @param reset @c TRUE resets the vbi_page dirty fields in cache after
+ *   fetching. Pass @c FALSE only if you plan to call this function again
  *   to update other displays.
  * 
- * Fetches a Closed Caption page designated by @pgno from the cache,
- * formats and stores it in @pg. CC pages are transmitted basically in
+ * Fetches a Closed Caption page designated by @a pgno from the cache,
+ * formats and stores it in @a pg. CC pages are transmitted basically in
  * two modes: at once and character by character ("roll-up" mode).
  * Either way you get a snapshot of the page as it should appear on
  * screen at present. With vbi_event_handler_add() you can request a
- * #VBI_EVENT_CAPTION event to be notified about pending changes
+ * @c VBI_EVENT_CAPTION event to be notified about pending changes
  * (in case of "roll-up" mode that is with each new word received)
- * and the #vbi_page dirty fields will mark the lines actually in
+ * and the vbi_page->dirty fields will mark the lines actually in
  * need of updates, to speed up rendering.
  * 
  * Although safe to do, this function is not supposed to be
- * called from an event handler since rendering may block decoding
+ * called from an event handler, since rendering may block decoding
  * for extended periods of time.
  * 
- * Return value:
- * FALSE if some error occured.
- **/
+ * @return
+ * @c FALSE if some error occured.
+ */
 vbi_bool
 vbi_fetch_cc_page(vbi_decoder *vbi, vbi_page *pg, vbi_pgno pgno, vbi_bool reset)
 {
-	channel *ch = vbi->cc.channel + ((pgno - 1) & 7);
+	cc_channel *ch = vbi->cc.channel + ((pgno - 1) & 7);
 	vbi_page *spg;
 
 	if (pgno < 1 || pgno > 8)
