@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: decoder.c,v 1.1.1.1 2002/01/12 16:18:36 mschimek Exp $ */
+/* $Id: decoder.c,v 1.13 2003/11/14 05:29:18 mschimek Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -25,8 +25,20 @@
 #include <assert.h>
 #include <pthread.h>
 
+#include "misc.h"
 #include "decoder.h"
 
+/**
+ * @addtogroup Rawdec Raw VBI decoder
+ * @ingroup Raw
+ * @brief Converting raw VBI samples to bits and bytes.
+ *
+ * The libzvbi already offers hardware interfaces to obtain sliced
+ * VBI data for further processing. However if you want to write your own
+ * interface or decode data services not covered by libzvbi you can use
+ * these lower level functions.
+ */
+
 /*
  *  Bit Slicer
  */
@@ -34,45 +46,55 @@
 #define OVERSAMPLING 4		/* 1, 2, 4, 8 */
 #define THRESH_FRAC 9
 
-/* note this is inlined, with bpp and endian const */
-static inline int
-sample(vbi_bit_slicer *d, uint8_t *raw,
-       int offs, int bpp, int endian)
+/*
+ * Note this is just a template. The code is inlined,
+ * with bpp and endian being const.
+ *
+ * This function translates from the image format to
+ * plain bytes, with linear interpolation of samples.
+ * Could be further improved with a lowpass filter.
+ */
+static inline unsigned int
+sample(uint8_t *raw, int offs, int bpp, int endian)
 {
 	unsigned char frac = offs;
 	int raw0, raw1;
 
 	switch (bpp) {
-	case 14: /* 1:5:5:5 LE */
+	case 14: /* 1:5:5:5 LE/BE */
 		raw += (offs >> 8) * 2;
 		raw0 = (raw[0 + endian] + raw[1 - endian] * 256) & 0x07C0;
 		raw1 = (raw[2 + endian] + raw[3 - endian] * 256) & 0x07C0;
 		return (raw1 - raw0) * frac + (raw0 << 8);
 
-	case 15: /* 5:5:5:1 LE */
+	case 15: /* 5:5:5:1 LE/BE */
 		raw += (offs >> 8) * 2;
 		raw0 = (raw[0 + endian] + raw[1 - endian] * 256) & 0x03E0;
 		raw1 = (raw[2 + endian] + raw[3 - endian] * 256) & 0x03E0;
 		return (raw1 - raw0) * frac + (raw0 << 8);
 
-	case 16: /* 5:6:5 LE */
+	case 16: /* 5:6:5 LE/BE */
 		raw += (offs >> 8) * 2;
 		raw0 = (raw[0 + endian] + raw[1 - endian] * 256) & 0x07E0;
 		raw1 = (raw[2 + endian] + raw[3 - endian] * 256) & 0x07E0;
 		return (raw1 - raw0) * frac + (raw0 << 8);
 
-	default:
+	default: /* 8 (intermediate bytes skipped by caller) */
 		raw += (offs >> 8) * bpp;
 		return (raw[bpp] - raw[0]) * frac + (raw[0] << 8);
 	}
 }
 
-/* note this is inlined, with bpp a const */
+/*
+ * Note this is just a template. The code is inlined,
+ * with bpp being const.
+ */
 static inline vbi_bool
 bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 		uint8_t *buf, int bpp, int endian)
 {
-	int i, j, k, cl = 0, thresh0 = d->thresh, tr;
+	unsigned int i, j, k;
+	unsigned int cl = 0, thresh0 = d->thresh, tr;
 	unsigned int c = 0, t;
 	unsigned char b, b1 = 0;
 	int raw0, raw1, mask;
@@ -108,7 +130,7 @@ bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 			} else {
 				cl += d->cri_rate;
 
-				if (cl >= d->oversampling_rate) {
+				if (cl >= (unsigned int) d->oversampling_rate) {
 					cl -= d->oversampling_rate;
 
 					c = c * 2 + b;
@@ -119,7 +141,7 @@ bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 						c = 0;
 
 						for (j = d->frc_bits; j > 0; j--) {
-							c = c * 2 + (sample(d, raw, i, bpp, endian) >= tr);
+							c = c * 2 + (sample(raw, i, bpp, endian) >= tr);
     							i += d->step;
 						}
 
@@ -131,9 +153,9 @@ bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 						
 						switch (d->endian) {
 						case 3:
-							for (j = 0; j < d->payload; j++) {
+							for (j = 0; j < (unsigned int) d->payload; j++) {
 					    			c >>= 1;
-								c += (sample(d, raw, i, bpp, endian) >= tr) << 7;
+								c += (sample(raw, i, bpp, endian) >= tr) << 7;
 			    					i += d->step;
 
 								if ((j & 7) == 7)
@@ -144,8 +166,8 @@ bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 							break;
 
 						case 2:
-							for (j = 0; j < d->payload; j++) {
-								c = c * 2 + (sample(d, raw, i, bpp, endian) >= tr);
+							for (j = 0; j < (unsigned int) d->payload; j++) {
+								c = c * 2 + (sample(raw, i, bpp, endian) >= tr);
 			    					i += d->step;
 
 								if ((j & 7) == 7)
@@ -159,7 +181,7 @@ bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 							for (j = d->payload; j > 0; j--) {
 								for (k = 0; k < 8; k++) {
 						    			c >>= 1;
-									c += (sample(d, raw, i, bpp, endian) >= tr) << 7;
+									c += (sample(raw, i, bpp, endian) >= tr) << 7;
 			    						i += d->step;
 								}
 
@@ -171,7 +193,7 @@ bit_slicer_tmpl(vbi_bit_slicer *d, uint8_t *raw,
 						case 0:
 							for (j = d->payload; j > 0; j--) {
 								for (k = 0; k < 8; k++) {
-									c = c * 2 + (sample(d, raw, i, bpp, endian) >= tr);
+									c = c * 2 + (sample(raw, i, bpp, endian) >= tr);
 			    						i += d->step;
 								}
 
@@ -266,42 +288,43 @@ bit_slicer_565_be(vbi_bit_slicer *d, uint8_t *raw, uint8_t *buf)
 }
 
 /**
- * vbi_bit_slicer_init:
- * @slicer: Pointer to #vbi_bit_slicer object to be initialized. 
- * @raw_samples: Number of samples or pixels in one raw vbi line
- *   later passed to vbi_bit_slice(). This sets a scan limit. 
- * @sampling_rate: Raw vbi sampling rate in Hz, that is the number of
+ * @param slicer Pointer to vbi_bit_slicer object to be initialized. 
+ * @param raw_samples Number of samples or pixels in one raw vbi line
+ *   later passed to vbi_bit_slice(). This limits the number of
+ *   bytes read from the sample buffer.
+ * @param sampling_rate Raw vbi sampling rate in Hz, that is the number of
  *   samples or pixels sampled per second by the hardware. 
- * @cri_rate: The Clock Run In is a NRZ modulated sequence of '0' and
- *   '1' bits prepending most data transmissions to synchronize
- *   data acquisition circuits. This parameter gives the CRI bit
+ * @param cri_rate The Clock Run In is a NRZ modulated
+ *   sequence of '0' and '1' bits prepending most data transmissions to
+ *   synchronize data acquisition circuits. This parameter gives the CRI bit
  *   rate in Hz, that is the number of CRI bits transmitted per second.
- * @bit_rate: The transmission bit rate of all data bits following the CRI
+ * @param bit_rate The transmission bit rate of all data bits following the CRI
  *   in Hz.
- * @cri_frc: The FRaming Code usually following the CRI is a bit sequence
+ * @param cri_frc The FRaming Code usually following the CRI is a bit sequence
  *   identifying the data service, and per libzvbi definition modulated
  *   and transmitted at the same bit rate as the payload (however nothing
  *   stops you from counting all nominal CRI and FRC bits as CRI).
  *   The bit slicer compares the bits in this word, lsb last transmitted,
  *   against the transmitted CRI and FRC. Decoding of payload starts
  *   with the next bit after a match.
- * @cri_mask: Of the CRI bits in @cri_frc, only these bits are
+ * @param cri_mask Of the CRI bits in @c cri_frc, only these bits are
  *   actually significant for a match. For instance it is wise
  *   not to rely on the very first CRI bits transmitted. Note this
- *   mask is not shifted left by @frc_bits.
- * @cri_bits: 
- * @frc_bits: Number of CRI and FRC bits in @cri_frc, respectively.
+ *   mask is not shifted left by @a frc_bits.
+ * @param cri_bits 
+ * @param frc_bits Number of CRI and FRC bits in @a cri_frc, respectively.
  *   Their sum is limited to 32.
- * @payload: Number of payload <emphasis>bits</>. Only this data
+ * @param payload Number of payload <em>bits</em>. Only this data
  *   will be stored in the vbi_bit_slice() output. If this number
  *   is no multiple of eight, the most significant bits of the
  *   last byte are undefined.
- * @modulation: Modulation of the vbi data, see #vbi_modulation.
- * @fmt: Format of the raw data, see #vbi_pixfmt.
+ * @param modulation Modulation of the vbi data, see vbi_modulation.
+ * @param fmt Format of the raw data, see vbi_pixfmt.
  * 
- * Initializes #vbi_bit_slicer object. Usually you will not use this
- * function, but the raw vbi decoder which handles all these details.
- **/
+ * Initializes vbi_bit_slicer object. Usually you will not use this
+ * function but vbi_raw_decode(), the vbi image decoder which handles
+ * all these details.
+ */
 void
 vbi_bit_slicer_init(vbi_bit_slicer *slicer,
 		    int raw_samples, int sampling_rate,
@@ -413,7 +436,7 @@ vbi_bit_slicer_init(vbi_bit_slicer *slicer,
 	slicer->frc			= cri_frc & f_mask;
 	slicer->frc_bits		= frc_bits;
 	/* Payload bit distance in 1/256 raw samples. */
-	slicer->step			= sampling_rate * 256.0 / bit_rate;
+	slicer->step			= (int)(sampling_rate * 256.0 / bit_rate);
 
 	if (payload & 7) {
 		slicer->payload	= payload;
@@ -427,27 +450,29 @@ vbi_bit_slicer_init(vbi_bit_slicer *slicer,
 	case VBI_MODULATION_NRZ_MSB:
 		slicer->endian--;
 	case VBI_MODULATION_NRZ_LSB:
-		slicer->phase_shift = sampling_rate * 256.0 / cri_rate * .5
-			+ sampling_rate * 256.0 / bit_rate * .5 + 128;
+		slicer->phase_shift = (int)
+			(sampling_rate * 256.0 / cri_rate * .5
+			 + sampling_rate * 256.0 / bit_rate * .5 + 128);
 		break;
 
 	case VBI_MODULATION_BIPHASE_MSB:
 		slicer->endian--;
 	case VBI_MODULATION_BIPHASE_LSB:
 		/* Phase shift between the NRZ modulated CRI and the rest */
-		slicer->phase_shift = sampling_rate * 256.0 / cri_rate * .5
-			+ sampling_rate * 256.0 / bit_rate * .25 + 128;
+		slicer->phase_shift = (int)
+			(sampling_rate * 256.0 / cri_rate * .5
+			 + sampling_rate * 256.0 / bit_rate * .25 + 128);
 		break;
 	}
 }
 
 /*
- *  Data Service Decoder
+ * Data Service Table
  */
 
 #define MAX_JOBS		(sizeof(((vbi_raw_decoder *) 0)->jobs)	\
 				 / sizeof(((vbi_raw_decoder *) 0)->jobs[0]))
-#define MAX_WAYS		4
+#define MAX_WAYS		8
 
 struct vbi_service_par {
 	unsigned int	id;		/* VBI_SLICED_ */
@@ -561,13 +586,13 @@ vbi_services[] = {
 };
 
 /**
- * vbi_sliced_name:
- * @service: VBI_SLICED_ value, for example from a #vbi_sliced structure.
+ * @param service A @ref VBI_SLICED_ symbol, for example from a vbi_sliced structure.
  *
- * Return value:
- * Name of the @service, ASCII, or %NULL if unknown.
- **/
-char *
+ * @return
+ * Name of the @a service, ASCII, or @c NULL if unknown.
+ */
+/* XXX make return type const */
+const char *
 vbi_sliced_name(unsigned int service)
 {
 	int i;
@@ -587,13 +612,26 @@ vbi_sliced_name(unsigned int service)
 	return NULL;
 }
 
+#ifndef DECODER_PATTERN_DUMP
+#define DECODER_PATTERN_DUMP 0
+#endif
+
+#ifndef DECODER_SERVICE_DIAG
+#define DECODER_SERVICE_DIAG 0
+#endif
+
+#define ds_diag(templ, args...)						\
+do { /* Check syntax in any case */					\
+	if (DECODER_SERVICE_DIAG)					\
+		fprintf (stderr, templ , ##args);			\
+} while (0)
+
 /**
- * vbi_raw_decode:
- * @rd: Initialized #vbi_raw_decoder structure.
- * @raw: A raw vbi image as defined in the #vbi_raw_decoder structure
+ * @param rd Initialized vbi_raw_decoder structure.
+ * @param raw A raw vbi image as defined in the vbi_raw_decoder structure
  *   (rd->sampling_format, rd->bytes_per_line, rd->count[0] + rd->count[1]
  *    scan lines).
- * @out: Buffer to store the decoded #vbi_sliced data. Since every
+ * @param out Buffer to store the decoded vbi_sliced data. Since every
  *   vbi scan line may contain data, this must be an array of vbi_sliced
  *   with the same number of entries as scan lines in the raw image
  *   (rd->count[0] + rd->count[1]).
@@ -604,11 +642,15 @@ vbi_sliced_name(unsigned int service)
  * Note this function attempts to learn which lines carry which data
  * service, or none, to speed up decoding. You should avoid using the same
  * vbi_raw_decoder structure for different sources.
- * 
- * Return value:
- * The number of lines decoded, i. e. the number of #vbi_sliced records
+ *
+ * @bug This function ignores the sampling_format field in struct
+ * vbi_raw_decoder, always assuming VBI_PIXFMT_YUV420.
+ *
+ * @return
+ * The number of lines decoded, i. e. the number of vbi_sliced records
  * written.
- **/
+ */
+/* XXX bit_slicer_1() */
 int
 vbi_raw_decode(vbi_raw_decoder *rd, uint8_t *raw, vbi_sliced *out)
 {
@@ -631,11 +673,27 @@ vbi_raw_decode(vbi_raw_decoder *rd, uint8_t *raw, vbi_sliced *out)
 		if (rd->interlaced && i == rd->count[0])
 			raw = raw1 + rd->bytes_per_line;
 
+		if (DECODER_PATTERN_DUMP) {
+			fprintf(stderr, "L%02d ", i);
+			for (j = 0; j < MAX_WAYS; j++)
+				if (pattern[j] < 1 || pattern[j] > 8)
+					fprintf(stderr, "0x%02x       ",
+						pattern[j] & 0xFF);
+				else
+					fprintf(stderr, "0x%08x ",
+						rd->jobs[pattern[j] - 1].id);
+			fprintf(stderr, "\n");
+		}
+
 		for (pat = pattern;; pat++) {
-			if ((j = *pat) > 0) {
+			j = *pat; /* data service n, blank 0, or counter -n */
+
+			if (j > 0) {
 				job = rd->jobs + (j - 1);
+
 				if (!bit_slicer_1(&job->slicer, raw + job->offset, out->data))
-					continue;
+					continue; /* no match, try next data service */
+
 				if (job->id == VBI_SLICED_WSS_CPR1204) {
 					const int poly = (1 << 6) + (1 << 1) + 1;
 					int crc, j;
@@ -649,30 +707,48 @@ vbi_raw_decode(vbi_raw_decoder *rd, uint8_t *raw, vbi_sliced *out)
 					}
 
 					if (crc)
-						continue;
+						continue; /* no match */
 				}
+
+				/* Positive match, output decoded line */
+
 				out->id = job->id;
+
 				if (i >= rd->count[0])
 					out->line = (rd->start[1] > 0) ? rd->start[1] - rd->count[0] + i : 0;
 				else
 					out->line = (rd->start[0] > 0) ? rd->start[0] + i : 0;
 				out++;
+
+				/* Predict line as non-blank, force testing for
+				   data services in the next 128 frames */
 				pattern[MAX_WAYS - 1] = -128;
+
 			} else if (pat == pattern) {
+				/* Line was predicted as blank, once in 16
+				   frames look for data services */
 				if (readj == 0) {
-					j = pattern[1];
-					pattern[1] = pattern[2];
-					pattern[2] = pattern[3];
-					pat += MAX_WAYS - 1;
+					j = pattern[0];
+					memmove (&pattern[0], &pattern[1],
+						 sizeof (*pattern) * (MAX_WAYS - 1));
+					pattern[MAX_WAYS - 1] = j;
 				}
+
+				break;
 			} else if ((j = pattern[MAX_WAYS - 1]) < 0) {
+				/* Increment counter, when zero predict line as
+				   blank and stop looking for data services */
 				pattern[MAX_WAYS - 1] = j + 1;
     				break;
+			} else {
+				/* found nothing, j = 0 */
 			}
 
+			/* Try the found data service first next time */
 			*pat = pattern[0];
 			pattern[0] = j;
-			break;
+
+			break; /* line done */
 		}
 
 		raw += pitch;
@@ -687,22 +763,99 @@ vbi_raw_decode(vbi_raw_decoder *rd, uint8_t *raw, vbi_sliced *out)
 }
 
 /**
- * vbi_raw_decoder_remove_services:
- * @rd: Initialized #vbi_raw_decoder structure.
- * @services: Set of VBI_SLICED_.
+ * @param rd Initialized vbi_raw_decoder structure.
+ * @param start Array of start line indices for both fields
+ * @param count Array of line counts for both fields
+ * 
+ * Grows or shrinks the internal state arrays for VBI geometry changes
+ */
+void
+vbi_raw_decoder_resize( vbi_raw_decoder *rd, int * start, unsigned int * count )
+{
+	int8_t * old_pattern;
+	int8_t * base;
+	int8_t * dest;
+	int8_t * src;
+	int      frm;
+	int      line;
+
+	if ( (rd->start[0] == start[0]) && (rd->start[1] == start[1]) &&
+	     (rd->count[0] == count[0]) && (rd->count[1] == count[1]) )
+		return;
+
+	if (rd->pattern != NULL) {
+		old_pattern = rd->pattern;
+
+		rd->pattern = (int8_t *) calloc((count[0] + count[1])
+						* MAX_WAYS, sizeof(rd->pattern[0]));
+
+		base = old_pattern;
+		for (frm = 0; frm < 2; frm++) {
+			for (line = rd->start[frm]; line < rd->start[frm] + rd->count[frm]; line++) {
+				if ((line >= start[frm]) && (line < start[frm] + count[frm])) {
+					dest = rd->pattern + (line - start[frm]) * MAX_WAYS;
+					src  = old_pattern + (line - rd->start[frm]) * MAX_WAYS;
+					memcpy(dest, src, MAX_WAYS * sizeof(*rd->pattern));
+				}
+			}
+			base += rd->count[frm] * MAX_WAYS;
+
+			rd->start[frm] = start[frm];
+			rd->count[frm] = count[frm];
+		}
+
+		free(old_pattern);
+        }
+}
+
+/**
+ *  Helper function for service removal:
+ *  remove one job from one VBI line's pattern array
+ */
+static void
+vbi_raw_decoder_remove_pattern( int job, int8_t * pattern, int pattern_size )
+{
+        int8_t * pat;
+	int ways_left;
+        int  i, j;
+
+        for (i = 0; i < pattern_size; i++) {
+                pat = pattern + i * MAX_WAYS;
+                for (j = 0; j < MAX_WAYS; j++) {
+                        if (pat[j] == job + 1) {
+                                ways_left = (MAX_WAYS - 1) - j;
+
+				if (ways_left > 0)
+                                        memmove(&pat[j], &pat[j + 1],
+                                                ways_left * sizeof(*pattern));
+
+                                pat[MAX_WAYS - 1] = 0;
+                        }
+                }
+        }
+
+        for (i = 0; i < pattern_size; i++) {
+                if (pattern[i] >= job + 1)
+                        pattern[i] -= 1;
+        }
+}
+
+/**
+ * @param rd Initialized vbi_raw_decoder structure.
+ * @param services Set of @ref VBI_SLICED_ symbols.
  * 
  * Removes one or more data services to be decoded from the
- * #vbi_raw_decoder structure. This function can be called at any
+ * vbi_raw_decoder structure. This function can be called at any
  * time and does not touch sampling parameters. 
  * 
- * Return value: 
- * Set of VBI_SLICED_ describing the remaining data services
- * that will be decoded.
- **/
+ * @return 
+ * Set of @ref VBI_SLICED_ symbols describing the remaining data
+ * services that will be decoded.
+ */
 unsigned int
 vbi_raw_decoder_remove_services(vbi_raw_decoder *rd, unsigned int services)
 {
-	int i, j;
+	int i;
 	int pattern_size = (rd->count[0] + rd->count[1]) * MAX_WAYS;
 
 	pthread_mutex_lock(&rd->mutex);
@@ -710,21 +863,14 @@ vbi_raw_decoder_remove_services(vbi_raw_decoder *rd, unsigned int services)
 	for (i = 0; i < rd->num_jobs;) {
 		if (rd->jobs[i].id & services) {
 			if (rd->pattern)
-				for (j = 0; j < pattern_size; j++)
-					if (rd->pattern[j] == i + 1) {
-						int ways_left = MAX_WAYS - 1
-							- j % MAX_WAYS;
+                                vbi_raw_decoder_remove_pattern(i, rd->pattern, pattern_size);
 
-						memmove(&rd->pattern[j], &rd->pattern[j + 1],
-							ways_left * sizeof(&rd->pattern[0]));
+                        if (i + 1 < MAX_JOBS)
+			        memmove(rd->jobs + i, rd->jobs + (i + 1),
+				        (MAX_JOBS - (i + 1)) * sizeof(rd->jobs[0]));
 
-						rd->pattern[j + ways_left] = 0;
-					}
-
-			memmove(rd->jobs + i, rd->jobs + (i + 1),
-				(MAX_JOBS - i - 1) * sizeof(rd->jobs[0]));
-
-			rd->jobs[--rd->num_jobs].id = 0;
+			rd->num_jobs -= 1;
+			rd->jobs[rd->num_jobs].id = 0;
 		} else
 			i++;
 	}
@@ -735,36 +881,189 @@ vbi_raw_decoder_remove_services(vbi_raw_decoder *rd, unsigned int services)
 }
 
 /**
- * vbi_raw_decoder_add_services:
- * @rd: Initialized #vbi_raw_decoder structure.
- * @services: Set of VBI_SLICED_.
- * @strict: A value of 0, 1 or 2 requests loose, reliable or strict
+ *  Helper function for service add and check functions:
+ *  check if the given service can be decoded with the parameters in rd;
+ *  if yes, return TRUE and line start and count for both fields within
+ *  the range limits of rd.
+ */
+static vbi_bool
+vbi_raw_decoder_check_service(const vbi_raw_decoder *rd, int srv_idx, int strict,
+                              int *row, int *count)
+{
+	double signal;
+	int field;
+	vbi_bool result = FALSE;
+
+	if (vbi_services[srv_idx].scanning != rd->scanning)
+		goto finished;
+
+	if ((vbi_services[srv_idx].id & (VBI_SLICED_CAPTION_525_F1
+				       | VBI_SLICED_CAPTION_525))
+	    && (rd->start[0] <= 0 || rd->start[1] <= 0)) {
+		/*
+		 *  The same format is used on other lines
+		 *  for non-CC data.
+		 */
+		goto finished;
+	}
+
+	signal = vbi_services[srv_idx].cri_bits / (double) vbi_services[srv_idx].cri_rate
+		 + (vbi_services[srv_idx].frc_bits + vbi_services[srv_idx].payload)
+		   / (double) vbi_services[srv_idx].bit_rate;
+
+	if (rd->offset > 0 && strict > 0) {
+		double offset = rd->offset / (double) rd->sampling_rate;
+		double samples_end = (rd->offset + rd->bytes_per_line)
+				     / (double) rd->sampling_rate;
+
+		if (offset > (vbi_services[srv_idx].offset / 1e9 - 0.5e-6)) {
+			ds_diag ("skipping service 0x%08X: H-Off %d = %f > %f\n",
+				vbi_services[srv_idx].id, rd->offset, offset,
+				vbi_services[srv_idx].offset / 1e9 - 0.5e-6);
+			goto finished;
+		}
+
+		if (samples_end < (vbi_services[srv_idx].offset / 1e9
+				   + signal + 0.5e-6)) {
+			ds_diag ("skipping service 0x%08X: sampling window too short: "
+				"end %f < %f = offset %d *10^-9 + %f\n",
+				vbi_services[srv_idx].id, samples_end,
+				vbi_services[srv_idx].offset / 1e9 + signal + 0.5e-6,
+				vbi_services[srv_idx].offset, signal);
+			goto finished;
+		}
+	} else {
+		double samples = rd->bytes_per_line
+				 / (double) rd->sampling_rate;
+
+		if (samples < (signal + 1.0e-6)) {
+			ds_diag ("skipping service 0x%08X: not enough samples\n",
+				vbi_services[srv_idx].id);
+			goto finished;
+		}
+	}
+
+	for (field = 0; field < 2; field++) {
+		int start = rd->start[field];
+		int end = start + rd->count[field] - 1;
+
+		if (!rd->synchronous) {
+			ds_diag ("skipping service 0x%08X: not sync'ed\n",
+				vbi_services[srv_idx].id);
+			goto finished; /* too difficult */
+		}
+
+		if (!(vbi_services[srv_idx].first[field] && vbi_services[srv_idx].last[field])) {
+			count[field] = 0;
+			continue;
+		}
+
+		if (rd->count[field] == 0) {
+			ds_diag ("skipping service 0x%08X: zero count\n",
+				vbi_services[srv_idx].id);
+			goto finished;
+		}
+
+		if (rd->start[field] > 0 && strict > 0) {
+			/*
+			 *  May succeed if not all scanning lines
+			 *  available for the service are actually used.
+			 */
+			if (strict > 1
+			    || (vbi_services[srv_idx].first[field] ==
+				vbi_services[srv_idx].last[field]))
+				if (start > vbi_services[srv_idx].first[field] ||
+				    end < vbi_services[srv_idx].last[field]) {
+					ds_diag ("skipping service 0x%08X: lines not available "
+						"have %d-%d, need %d-%d\n",
+						vbi_services[srv_idx].id, start, end,
+						vbi_services[srv_idx].first[field],
+						vbi_services[srv_idx].last[field]);
+					goto finished;
+				}
+
+			row[field] = MAX(0, (int) vbi_services[srv_idx].first[field] - start);
+			count[field] = MIN(end, vbi_services[srv_idx].last[field])
+				       - (start + row[field]) + 1;
+		} else {
+			row[field] = 0;
+			count[field] = rd->count[field];
+		}
+	}
+	row[1] += rd->count[0];
+
+	result = TRUE;
+
+finished:
+	return result;
+}
+
+
+/**
+ * @param rd Initialized vbi_raw_decoder structure.
+ * @param services Set of @ref VBI_SLICED_ symbols.
+ * @param strict See description of vbi_raw_decoder_add_services()
+ *
+ * Check which of the given services can be decoded with current capture
+ * parameters at a given strictness level.
+ *
+ * @return
+ * Subset of services actually decodable.
+ */
+unsigned int
+vbi_raw_decoder_check_services(vbi_raw_decoder *rd, unsigned int services, int strict)
+{
+	int row[2], count[2];
+	int i;
+
+	services &= ~(VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625);
+
+	for (i = 0; vbi_services[i].id; i++) {
+
+		if ( ((vbi_services[i].id & services) != 0) &&
+		     (vbi_raw_decoder_check_service(rd, i, strict, row, count) == FALSE) ) {
+
+			/* incompatible service */
+			services &= ~ vbi_services[i].id;
+		}
+	}
+	return services;
+}
+
+
+/**
+ * @param rd Initialized vbi_raw_decoder structure.
+ * @param services Set of @ref VBI_SLICED_ symbols.
+ * @param strict A value of 0, 1 or 2 requests loose, reliable or strict
  *  matching of sampling parameters. For example if the data service
- *  requires knowledge of line numbers while they are not known, %0
+ *  requires knowledge of line numbers while they are not known, @c 0
  *  will accept the service (which may work if the scan lines are
- *  occupied in a non-confusing way) but %1 or %2 will not. If the
- *  data service may use more lines than sampled, %1 will accept
- *  but %2 will not. If unsure, set to %1.
+ *  populated in a non-confusing way) but @c 1 or @c 2 will not. If the
+ *  data service <i>may</i> use more lines than are sampled, @c 1 will
+ *  accept but @c 2 will not. If unsure, set to @c 1.
  * 
- * After you initialized the sampling parameters in @rd (according to
+ * After you initialized the sampling parameters in @a rd (according to
  * the abilities of your raw vbi source), this function adds one or more
  * data services to be decoded. The libzvbi raw vbi decoder can decode up
  * to eight data services in parallel. You can call this function while
  * already decoding, it does not change sampling parameters and you must
  * not change them either after calling this.
  * 
- * Return value:
- * Set of VBI_SLICED_ describing the data services that actually will
- * be decoded. This excludes those services not decodable given
- * the sampling parameters in @rd.
- **/
+ * @return
+ * Set of @ref VBI_SLICED_ symbols describing the data services that actually
+ * will be decoded. This excludes those services not decodable given
+ * the sampling parameters in @a rd.
+ */
 unsigned int
 vbi_raw_decoder_add_services(vbi_raw_decoder *rd, unsigned int services, int strict)
 {
 	double off_min = (rd->scanning == 525) ? 7.9e-6 : 8.0e-6;
-	int row[2], count[2], way;
+	double offset = rd->offset / (double) rd->sampling_rate;
 	struct _vbi_raw_decoder_job *job;
 	int8_t *pattern;
+	int row[2], count[2];
+	int skip;
+	int way;
 	int i, j, k;
 
 	pthread_mutex_lock(&rd->mutex);
@@ -772,57 +1071,18 @@ vbi_raw_decoder_add_services(vbi_raw_decoder *rd, unsigned int services, int str
 	services &= ~(VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625);
 
 	if (!rd->pattern)
-		rd->pattern = calloc((rd->count[0] + rd->count[1])
-				      * MAX_WAYS, sizeof(rd->pattern[0]));
+		rd->pattern = (int8_t *) calloc((rd->count[0] + rd->count[1])
+						* MAX_WAYS, sizeof(rd->pattern[0]));
 
 	for (i = 0; vbi_services[i].id; i++) {
-		double signal;
-		int skip = 0;
-
-		if (rd->num_jobs >= MAX_JOBS)
+		if (rd->num_jobs >= (int) MAX_JOBS)
 			break;
 
-		if (!(vbi_services[i].id & services))
+		if ((vbi_services[i].id & services) == 0)
 			continue;
 
-		if (vbi_services[i].scanning != rd->scanning)
-			goto eliminate;
-
-		if ((vbi_services[i].id & (VBI_SLICED_CAPTION_525_F1
-					   | VBI_SLICED_CAPTION_525))
-		    && (rd->start[0] <= 0 || rd->start[1] <= 0)) {
-			/*
-			 *  The same format is used on other lines
-			 *  for non-CC data.
-			 */
-			goto eliminate;
-		}
-
-		signal = vbi_services[i].cri_bits / (double) vbi_services[i].cri_rate
-			 + (vbi_services[i].frc_bits + vbi_services[i].payload)
-			   / (double) vbi_services[i].bit_rate;
-
-		if (rd->offset > 0 && strict > 0) {
-			double offset = rd->offset / (double) rd->sampling_rate;
-			double samples_end = (rd->offset + rd->bytes_per_line)
-					     / (double) rd->sampling_rate;
-
-			if (offset > (vbi_services[i].offset / 1e9 - 0.5e-6))
-				goto eliminate;
-
-			if (samples_end < (vbi_services[i].offset / 1e9
-					   + signal + 0.5e-6))
-				goto eliminate;
-
-			if (offset < off_min) /* skip colour burst */
-				skip = off_min * rd->sampling_rate;
-		} else {
-			double samples = rd->bytes_per_line
-				         / (double) rd->sampling_rate;
-
-			if (samples < (signal + 1.0e-6))
-				goto eliminate;
-		}
+		if (vbi_raw_decoder_check_service(rd, i, strict, row, count) == FALSE)
+			goto finished;
 
 		for (j = 0, job = rd->jobs; j < rd->num_jobs; job++, j++) {
 			unsigned int id = job->id | vbi_services[i].id;
@@ -839,41 +1099,6 @@ vbi_raw_decoder_add_services(vbi_raw_decoder *rd, unsigned int services, int str
 		}
 
 		for (j = 0; j < 2; j++) {
-			int start = rd->start[j];
-			int end = start + rd->count[j] - 1;
-
-			if (!rd->synchronous)
-				goto eliminate; /* too difficult */
-
-			if (!(vbi_services[i].first[j] && vbi_services[i].last[j])) {
-				count[j] = 0;
-				continue;
-			}
-
-			if (rd->count[j] == 0)
-				goto eliminate;
-
-			if (rd->start[j] > 0 && strict > 0) {
-				/*
-				 *  May succeed if not all scanning lines
-				 *  available for the service are actually used.
-				 */
-				if (strict > 1
-				    || (vbi_services[i].first[j] ==
-					vbi_services[i].last[j]))
-					if (start > vbi_services[i].first[j] ||
-					    end < vbi_services[i].last[j])
-						goto eliminate;
-
-				row[j] = MAX(0, (int) vbi_services[i].first[j] - start);
-				count[j] = MIN(end, vbi_services[i].last[j]) - start + 1;
-			} else {
-				row[j] = 0;
-				count[j] = rd->count[j];
-			}
-
-			row[1] += rd->count[0];
-
 			for (pattern = rd->pattern + row[j] * MAX_WAYS, k = count[j];
 			     k > 0; pattern += MAX_WAYS, k--) {
 				int free = 0;
@@ -883,19 +1108,32 @@ vbi_raw_decoder_add_services(vbi_raw_decoder *rd, unsigned int services, int str
 						 || ((pattern[way] - 1)
 						     == job - rd->jobs));
 
-				if (free <= 1) /* reserve one NULL way */
-					goto eliminate;
+				if (free <= 1) { /* reserve one NULL way */
+					ds_diag ("skipping service 0x%08X: no more patterns free\n",
+						vbi_services[i].id);
+					goto finished;
+				}
 			}
 		}
 
-		for (j = 0; j < 2; j++)
+		for (j = 0; j < 2; j++) {
 			for (pattern = rd->pattern + row[j] * MAX_WAYS, k = count[j];
 			     k > 0; pattern += MAX_WAYS, k--) {
 				for (way = 0; pattern[way] > 0
 				      && (pattern[way] - 1) != (job - rd->jobs); way++);
+
+                                assert((pattern + MAX_WAYS - rd->pattern) <= (rd->count[0] + rd->count[1]) * MAX_WAYS);
+
 				pattern[way] = (job - rd->jobs) + 1;
 				pattern[MAX_WAYS - 1] = -128;
 			}
+                }
+
+		/* skip colour burst */
+		if (rd->offset > 0 && strict > 0 && offset < off_min)
+			skip = (int)(off_min * rd->sampling_rate);
+		else
+			skip = 0;
 
 		job->id |= vbi_services[i].id;
 		job->offset = skip;
@@ -917,7 +1155,7 @@ vbi_raw_decoder_add_services(vbi_raw_decoder *rd, unsigned int services, int str
 			rd->num_jobs++;
 
 		rd->services |= vbi_services[i].id;
-eliminate:
+finished:
 		;
 	}
 
@@ -927,32 +1165,31 @@ eliminate:
 }
 
 /**
- * vbi_raw_decoder_parameters:
- * @rd: Initialized #vbi_raw_decoder structure.
- * @services: Set of VBI_SLICED_. Here (and only here) you can
- *   add #VBI_SLICED_VBI_625 or #VBI_SLICED_VBI_525 to include all
+ * @param rd Initialized vbi_raw_decoder structure.
+ * @param services Set of VBI_SLICED_ symbols. Here (and only here) you
+ *   can add @c VBI_SLICED_VBI_625 or @c VBI_SLICED_VBI_525 to include all
  *   vbi scan lines in the calculated sampling parameters.
- * @scanning: When 525 accept only NTSC services, when 625
+ * @param scanning When 525 accept only NTSC services, when 625
  *   only PAL/SECAM services. When scanning is 0, assume the scanning
  *   from the requested services, an ambiguous set will pick
  *   a 525 or 625 line system at random.
- * @max_rate: If given, the highest data bit rate in Hz of all
+ * @param max_rate If given, the highest data bit rate in Hz of all
  *   services requested is stored here. (The sampling rate
  *   should be at least twice as high; rd->sampling_rate will
  *   be set to a more reasonable value of 27 MHz derived
  *   from ITU-R Rec. 601.)
  * 
- * Calculate the sampling parameters in @rd required to receive and
- * decode the requested data @services. rd->sampling_format will be
- * #VBI_PIXFMT_YUV420, rd->bytes_per_line set accordingly to a
+ * Calculate the sampling parameters in @a rd required to receive and
+ * decode the requested data @a services. rd->sampling_format will be
+ * @c VBI_PIXFMT_YUV420, rd->bytes_per_line set accordingly to a
  * reasonable minimum. This function can be used to initialize hardware
  * prior to calling vbi_raw_decoder_add_service().
  * 
- * Return value:
- * Set of VBI_SLICED_ describing the data services covered by the
- * calculated sampling parameters. This excludes services the libzvbi
+ * @return
+ * Set of @ref VBI_SLICED_ symbols describing the data services covered
+ * by the calculated sampling parameters. This excludes services the libzvbi
  * raw decoder cannot decode.
- **/
+ */
 unsigned int
 vbi_raw_decoder_parameters(vbi_raw_decoder *rd, unsigned int services,
 			   int scanning, int *max_rate)
@@ -965,7 +1202,7 @@ vbi_raw_decoder_parameters(vbi_raw_decoder *rd, unsigned int services,
 	rd->sampling_format = VBI_PIXFMT_YUV420;
 	rd->sampling_rate = 27000000;	/* ITU-R Rec. 601 */
 	rd->bytes_per_line = 0;
-	rd->offset = 1000e-6 * rd->sampling_rate;
+	rd->offset = (int)(1000e-6 * rd->sampling_rate);
 	rd->start[0] = 1000;
 	rd->count[0] = 0;
 	rd->start[1] = 1000;
@@ -1004,10 +1241,10 @@ vbi_raw_decoder_parameters(vbi_raw_decoder *rd, unsigned int services,
 			 + (vbi_services[i].frc_bits + vbi_services[i].payload)
 			   / (double) vbi_services[i].bit_rate;
 
-		offset = (vbi_services[i].offset / 1e9 - left_margin)
-			 * rd->sampling_rate + 0.5;
-		samples = (signal + left_margin + 1.0e-6)
-			  * rd->sampling_rate + 0.5;
+		offset = (int)((vbi_services[i].offset / 1e9 - left_margin)
+			* rd->sampling_rate + 0.5);
+		samples = (int)((signal + left_margin + 1.0e-6)
+			  * rd->sampling_rate + 0.5);
 
 		rd->offset = MIN(rd->offset, offset);
 
@@ -1044,14 +1281,13 @@ vbi_raw_decoder_parameters(vbi_raw_decoder *rd, unsigned int services,
 }
 
 /**
- * vbi_raw_decoder_reset:
- * @rd: Initialized #vbi_raw_decoder structure.
+ * @param rd Initialized vbi_raw_decoder structure.
  * 
- * Reset a #vbi_raw_decoder structure. This removes
+ * Reset a vbi_raw_decoder structure. This removes
  * all previously added services to be decoded (if any)
  * but does not touch the sampling parameters. You are
  * free to change the sampling parameters after calling this.
- **/
+ */
 void
 vbi_raw_decoder_reset(vbi_raw_decoder *rd)
 {
@@ -1074,12 +1310,11 @@ vbi_raw_decoder_reset(vbi_raw_decoder *rd)
 }
 
 /**
- * vbi_raw_decoder_destroy:
- * @rd: Pointer to initialized #vbi_raw_decoder
- *  structure, can be %NULL.
+ * @param rd Pointer to initialized vbi_raw_decoder
+ *  structure, can be @c NULL.
  * 
- * Free all resources associated with @rd.
- **/
+ * Free all resources associated with @a rd.
+ */
 void
 vbi_raw_decoder_destroy(vbi_raw_decoder *rd)
 {
@@ -1089,11 +1324,10 @@ vbi_raw_decoder_destroy(vbi_raw_decoder *rd)
 }
 
 /**
- * vbi_raw_decoder_init:
- * @rd: Pointer to a #vbi_raw_decoder structure.
+ * @param rd Pointer to a vbi_raw_decoder structure.
  * 
- * Initializes a #vbi_raw_decoder structure.
- **/
+ * Initializes a vbi_raw_decoder structure.
+ */
 void
 vbi_raw_decoder_init(vbi_raw_decoder *rd)
 {
