@@ -22,7 +22,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: exp-gfx.c,v 1.2 2002/01/21 07:57:10 mschimek Exp $ */
+/* $Id: exp-gfx.c,v 1.7 2002/10/22 04:42:40 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "../config.h"
@@ -92,18 +92,18 @@ init_gfx(void)
 }
 
 /**
- * unicode_wstfont2:
- * @c: Unicode.
- * @italic: TRUE to switch to slanted character set (doesn't affect
+ * @internal
+ * @param c Unicode.
+ * @param italic @c TRUE to switch to slanted character set (doesn't affect
  *          Hebrew and Arabic). If this is a G1 block graphic character
  *          switch to separated block mosaic set.
  * 
  * Translate Unicode character to glyph number in wstfont2 image. 
  * 
- * Return value:
+ * @return
  * Glyph number.
- **/
-static int
+ */
+static unsigned int
 unicode_wstfont2(unsigned int c, int italic)
 {
 	static const unsigned short specials[] = {
@@ -113,16 +113,17 @@ unicode_wstfont2(unsigned int c, int italic)
 		0x20A0, 0x2030, 0x20AA, 0x2122, 0x2126, 0x215B, 0x215C, 0x215D,
 		0x215E, 0x2190, 0x2191, 0x2192, 0x2193, 0x25A0, 0x266A, 0xE800,
 		0xE75F };
-	int i;
+	const unsigned int invalid = 357;
+	unsigned int i;
 
 	if (c < 0x0180) {
 		if (c < 0x0080) {
 			if (c < 0x0020)
-				return 357; /* invalid */
+				return invalid;
 			else /* %3 Basic Latin (ASCII) 0x0020 ... 0x007F */
 				c = c - 0x0020 + 0 * 32;
 		} else if (c < 0x00A0)
-			return 357; /* invalid */
+			return invalid;
 		else /* %3 Latin-1 Supplement, Latin Extended-A 0x00A0 ... 0x017F */
 			c = c - 0x00A0 + 3 * 32;
 	} else if (c < 0xEE00) {
@@ -133,17 +134,17 @@ unicode_wstfont2(unsigned int c, int italic)
 				else /* %5 Greek 0x0370 ... 0x03CF */
 					c = c - 0x0370 + 12 * 32;
 			} else if (c < 0x0400)
-				return 357; /* invalid */
+				return invalid;
 			else /* %5 Cyrillic 0x0400 ... 0x045F */
 				c = c - 0x0400 + 15 * 32;
 		} else if (c < 0x0620) {
 			if (c < 0x05F0) {
 				if (c < 0x05D0)
-					return 357; /* invalid */
+					return invalid;
 				else /* %6 Hebrew 0x05D0 ... 0x05EF */
 					return c - 0x05D0 + 18 * 32;
 			} else if (c < 0x0600)
-				return 357; /* invalid */
+				return invalid;
 			else /* %6 Arabic 0x0600 ... 0x061F */
 				return c - 0x0600 + 19 * 32;
 		} else if (c >= 0xE600 && c < 0xE740)
@@ -155,7 +156,7 @@ unicode_wstfont2(unsigned int c, int italic)
 	} else if (c < 0xF000) { /* %4 G3 Graphics */
 		return c - 0xEF20 + 27 * 32;
 	} else /* 0xF000 ... 0xF7FF reserved for DRCS */
-		return 357; /* invalid */
+		return invalid;
 
 	if (italic)
 		return c + 31 * 32;
@@ -170,20 +171,20 @@ special:
 				return i + 10 * 32;
 		}
 
-	return 357; /* invalid */
+	return invalid;
 }
 
 /**
- * unicode_ccfont2:
- * @c: Unicode.
- * @italic: TRUE to switch to slanted character set.
+ * @internal
+ * @param c Unicode.
+ * @param italic @c TRUE to switch to slanted character set.
  * 
  * Translate Unicode character to glyph number in ccfont2 image. 
  * 
- * Return value:
+ * @return
  * Glyph number.
- **/
-static int
+ */
+static unsigned int
 unicode_ccfont2(unsigned int c, int italic)
 {
 	static const unsigned short specials[] = {
@@ -191,7 +192,7 @@ unicode_ccfont2(unsigned int c, int italic)
 		0x00ED, 0x00F3, 0x00FA, 0x00E7, 0x00F7, 0x00D1, 0x00F1, 0x25A0,
 		0x00AE, 0x00B0, 0x00BD, 0x00BF, 0x2122, 0x00A2, 0x00A3, 0x266A,
 		0x00E0, 0x0020, 0x00E8, 0x00E2, 0x00EA, 0x00EE, 0x00F4, 0x00FB };
-	int i;
+	unsigned int i;
 
 	if (c < 0x0020)
 		c = 15; /* invalid */
@@ -215,56 +216,56 @@ slant:
 }
 
 /**
- * peek:
- * @p: Plane of @canvas_type char, short, int.
- * @i: Index.
+ * @internal
+ * @param p Plane of @a canvas_type char, short, int.
+ * @param i Index.
  *
- * Return value:
- * Pixel @i in plane @p.
- **/
+ * @return
+ * Pixel @a i in plane @a p.
+ */
 #define peek(p, i)							\
 ((canvas_type == sizeof(uint8_t)) ? ((uint8_t *)(p))[i] :		\
     ((canvas_type == sizeof(uint16_t)) ? ((uint16_t *)(p))[i] :		\
 	((uint32_t *)(p))[i]))
 
 /**
- * poke:
- * @p: Plane of @canvas_type char, short, int.
- * @i: Index.
- * @v: Value.
+ * @internal
+ * @param p Plane of @a canvas_type char, short, int.
+ * @param i Index.
+ * @param v Value.
  * 
- * Set pixel @i in plane @p to value @v.
- **/
+ * Set pixel @a i in plane @a p to value @a v.
+ */
 #define poke(p, i, v)							\
 ((canvas_type == sizeof(uint8_t)) ? (((uint8_t *)(p))[i] = (v)) :	\
     ((canvas_type == sizeof(uint16_t)) ? (((uint16_t *)(p))[i] = (v)) :	\
 	(((uint32_t *)(p))[i] = (v))))
 
 /**
- * draw_char:
- * @canvas_type: sizeof(char, short, int).
- * @canvas: Pointer to image plane where the character is to be drawed.
- * @rowstride: @canvas <emphasis>byte</> distance from line to line.
- * @pen: Pointer to color palette of @canvas_type (index 0 background
+ * @internal
+ * @param canvas_type sizeof(char, short, int).
+ * @param canvas Pointer to image plane where the character is to be drawed.
+ * @param rowstride @a canvas <em>byte</em> distance from line to line.
+ * @param pen Pointer to color palette of @a canvas_type (index 0 background
  *   pixels, index 1 foreground pixels).
- * @font: Pointer to font image with width @cpl x @cw pixels, height @ch
- *   pixels, depth one bit, bit '1' is foreground.
- * @cpl: Chars per line (number of characters in @font image).
- * @cw: Character cell width in pixels.
- * @ch: Character cell height in pixels.
- * @glyph: Glyph number in font image, 0 ... @cpl - 1.
- * @bold: Draw character bold (font image | font image << 1).
- * @underline: Bit mask of character rows. For each bit
- *   1 << (n = 0 ... @ch - 1) set all of character row n to
+ * @param font Pointer to font image with width @a cpl x @a cw pixels, height
+ *   @a ch pixels, depth one bit, bit '1' is foreground.
+ * @param cpl Chars per line (number of characters in @a font image).
+ * @param cw Character cell width in pixels.
+ * @param ch Character cell height in pixels.
+ * @param glyph Glyph number in font image, 0 ... @a cpl - 1.
+ * @param bold Draw character bold (font image | font image << 1).
+ * @param underline Bit mask of character rows. For each bit
+ *   1 << (n = 0 ... @a ch - 1) set all of character row n to
  *   foreground color.
- * @size: Size of character, either NORMAL, DOUBLE_WIDTH (draws left
+ * @param size Size of character, either NORMAL, DOUBLE_WIDTH (draws left
  *   and right half), DOUBLE_HEIGHT (draws upper half only),
  *   DOUBLE_SIZE (left and right upper half), DOUBLE_HEIGHT2
  *   (lower half), DOUBLE_SIZE2 (left and right lower half).
  * 
  * Draw one character (function template - define a static version with
- * constant @canvas_type, @font, @cpl, @cw, @ch).
- **/
+ * constant @a canvas_type, @a font, @a cpl, @a cw, @a ch).
+ */
 static inline void
 draw_char(int canvas_type, uint8_t *canvas, int rowstride,
 	  uint8_t *pen, uint8_t *font, int cpl, int cw, int ch,
@@ -366,26 +367,26 @@ draw_char(int canvas_type, uint8_t *canvas, int rowstride,
 }
 
 /**
- * draw_drcs:
- * @canvas_type: sizeof(char, short, int).
- * @canvas: Pointer to image plane where the character is to be drawed.
- * @rowstride: @canvas <emphasis>byte</> distance from line to line.
- * @pen: Pointer to color palette of @canvas_type (index 0 ... 1 for
+ * @internal
+ * @param canvas_type sizeof(char, short, int).
+ * @param canvas Pointer to image plane where the character is to be drawed.
+ * @param rowstride @a canvas <em>byte</em> distance from line to line.
+ * @param pen Pointer to color palette of @a canvas_type (index 0 ... 1 for
  *   depth 1 DRCS, 0 ... 3 for depth 2, 0 ... 15 for depth 4).
- * @color: Offset into color palette.
- * @font: Pointer to DRCS image. Each pixel is coded in four bits, an
+ * @param color Offset into color palette.
+ * @param font Pointer to DRCS image. Each pixel is coded in four bits, an
  *   index into the color palette, and stored in LE order (i. e. first
  *   pixel 0x0F, second pixel 0xF0). Character size is 12 x 10 pixels,
  *   60 bytes, without padding.
- * @glyph: Glyph number in font image, 0x00 ... 0x3F.
- * @size: Size of character, either NORMAL, DOUBLE_WIDTH (draws left
+ * @param glyph Glyph number in font image, 0x00 ... 0x3F.
+ * @param size Size of character, either NORMAL, DOUBLE_WIDTH (draws left
  *   and right half), DOUBLE_HEIGHT (draws upper half only),
  *   DOUBLE_SIZE (left and right upper half), DOUBLE_HEIGHT2
  *   (lower half), DOUBLE_SIZE2 (left and right lower half).
  * 
  * Draw one Teletext Dynamically Redefinable Character (function template -
- * define a static version with constant @canvas_type, @font).
- **/
+ * define a static version with constant @a canvas_type, @a font).
+ */
 static inline void
 draw_drcs(int canvas_type, uint8_t *canvas, unsigned int rowstride,
 	  uint8_t *pen, int color, uint8_t *font, int glyph, vbi_size size)
@@ -461,16 +462,16 @@ draw_drcs(int canvas_type, uint8_t *canvas, unsigned int rowstride,
 }
 
 /**
- * draw_blank:
- * @canvas_type: sizeof(char, short, int).
- * @canvas: Pointer to image plane where the character is to be drawed.
- * @rowstride: @canvas <emphasis>byte</> distance from line to line.
- * @color: Color value of @canvas_type.
- * @cw: Character width in pixels.
- * @ch: Character height in pixels.
+ * @internal
+ * @param canvas_type sizeof(char, short, int).
+ * @param canvas Pointer to image plane where the character is to be drawed.
+ * @param rowstride @a canvas <em>byte</em> distance from line to line.
+ * @param color Color value of @a canvas_type.
+ * @param cw Character width in pixels.
+ * @param ch Character height in pixels.
  * 
  * Draw blank character.
- **/
+ */
 static inline void
 draw_blank(int canvas_type, uint8_t *canvas, unsigned int rowstride,
 	   unsigned int color, int cw, int ch)
@@ -486,21 +487,19 @@ draw_blank(int canvas_type, uint8_t *canvas, unsigned int rowstride,
 }
 
 /**
- * vbi_draw_cc_page_region:
- * @pg: Source page.
- * @fmt: Target format. For now only VBI_PIXFMT_RGBA32_LE (#vbi_rgba) permitted.
- * @canvas: Pointer to destination image (currently an array of vbi_rgba), this
- *   must be at least @rowstride * @height * 26 bytes large.
- * @rowstride: @canvas <emphasis>byte</> distance from line to line.
+ * @param fmt Target format. For now only VBI_PIXFMT_RGBA32_LE (vbi_rgba) permitted.
+ * @param canvas Pointer to destination image (currently an array of vbi_rgba), this
+ *   must be at least @a rowstride * @a height * 26 bytes large.
+ * @param rowstride @a canvas <em>byte</em> distance from line to line.
  *   If this is -1, pg->columns * 16 * sizeof(vbi_rgba) bytes will be assumed.
- * @column: First source column, 0 ... pg->columns - 1.
- * @row: First source row, 0 ... pg->rows - 1.
- * @width: Number of columns to draw, 1 ... pg->columns.
- * @height: Number of rows to draw, 1 ... pg->rows.
+ * @param column First source column, 0 ... pg->columns - 1.
+ * @param row First source row, 0 ... pg->rows - 1.
+ * @param width Number of columns to draw, 1 ... pg->columns.
+ * @param height Number of rows to draw, 1 ... pg->rows.
  * 
- * Draw a subsection of a Closed Caption #vbi_page. In this mode one
+ * Draw a subsection of a Closed Caption vbi_page. In this mode one
  * character occupies 16 x 26 pixels.
- **/
+ */
 void
 vbi_draw_cc_page_region(vbi_page *pg,
 			vbi_pixfmt fmt, void *canvas, int rowstride,
@@ -554,25 +553,24 @@ vbi_draw_cc_page_region(vbi_page *pg,
 }
 
 /**
- * vbi_draw_vt_page_region:
- * @pg: Source page.
- * @fmt: Target format. For now only VBI_PIXFMT_RGBA32_LE (#vbi_rgba) permitted.
- * @canvas: Pointer to destination image (currently an array of vbi_rgba), this
- *   must be at least @rowstride * @height * 10 bytes large.
- * @rowstride: @canvas <emphasis>byte</> distance from line to line.
+ * @param pg Source page.
+ * @param fmt Target format. For now only VBI_PIXFMT_RGBA32_LE (vbi_rgba) permitted.
+ * @param canvas Pointer to destination image (currently an array of vbi_rgba), this
+ *   must be at least @a rowstride * @a height * 10 bytes large.
+ * @param rowstride @a canvas <em>byte</em> distance from line to line.
  *   If this is -1, pg->columns * 12 * sizeof(vbi_rgba) bytes will be assumed.
- * @column: First source column, 0 ... pg->columns - 1.
- * @row: First source row, 0 ... pg->rows - 1.
- * @width: Number of columns to draw, 1 ... pg->columns.
- * @height: Number of rows to draw, 1 ... pg->rows.
- * @reveal: If FALSE, draw characters flagged 'concealed' (see #vbi_char) as
+ * @param column First source column, 0 ... pg->columns - 1.
+ * @param row First source row, 0 ... pg->rows - 1.
+ * @param width Number of columns to draw, 1 ... pg->columns.
+ * @param height Number of rows to draw, 1 ... pg->rows.
+ * @param reveal If FALSE, draw characters flagged 'concealed' (see vbi_char) as
  *   space (U+0020).
- * @flash_on: If FALSE, draw characters flagged 'blink' (see #vbi_char) as
+ * @param flash_on If FALSE, draw characters flagged 'blink' (see vbi_char) as
  *   space (U+0020).
  * 
- * Draw a subsection of a Teletext #vbi_page. In this mode one
+ * Draw a subsection of a Teletext vbi_page. In this mode one
  * character occupies 12 x 10 pixels.
- **/
+ */
 void
 vbi_draw_vt_page_region(vbi_page *pg,
 			vbi_pixfmt fmt, void *canvas, int rowstride,
@@ -581,7 +579,7 @@ vbi_draw_vt_page_region(vbi_page *pg,
 {
 	vbi_rgba pen[64], *canvast = canvas;
 	int count, row_adv;
-	int conceal, unicode;
+	int conceal, off, unicode;
 	vbi_char *ac;
 	int i;
 
@@ -606,6 +604,7 @@ vbi_draw_vt_page_region(vbi_page *pg,
 	row_adv = rowstride * 10 - width * 12 * sizeof(*canvast);
 
 	conceal = !reveal;
+	off = !flash_on;
 
 	if (pg->drcs_clut)
 		for (i = 2; i < 2 + 8 + 32; i++)
@@ -615,7 +614,7 @@ vbi_draw_vt_page_region(vbi_page *pg,
 		ac = &pg->text[row * pg->columns + column];
 
 		for (count = width; count > 0; count--, ac++) {
-			if ((ac->conceal & conceal) || !flash_on)
+			if ((ac->conceal & conceal) || (ac->flash & off))
 				unicode = 0x0020;
 			else
 				unicode = ac->unicode;
@@ -660,12 +659,13 @@ vbi_draw_vt_page_region(vbi_page *pg,
  */
 
 /**
- * vbi_get_max_rendered_size:
- * @w:
- * @h:
+ * @param w
+ * @param h
  *
- * Deprecated, will be removed.
- **/
+ * @deprecated
+ * Character cells are 12 x 10 for Teletext and 16 x 26 for Caption.
+ * Page size is in vbi_page.
+ */
 void
 vbi_get_max_rendered_size(int *w, int *h)
 {
@@ -674,12 +674,12 @@ vbi_get_max_rendered_size(int *w, int *h)
 }
 
 /**
- * vbi_get_vt_cell_size:
- * @w:
- * @h:
+ * @param w
+ * @param h
  *
- * Deprecated, will be removed.
- **/
+ * @deprecated
+ * Character cells are 12 x 10 for Teletext and 16 x 26 for Caption.
+ */
 void
 vbi_get_vt_cell_size(int *w, int *h)
 {
@@ -740,7 +740,7 @@ gfx_options[] = {
 static vbi_option_info *
 option_enum(vbi_export *e, int index)
 {
-	if (index < 0 || index >= elements(gfx_options))
+	if (index < 0 || index >= (int) elements(gfx_options))
 		return NULL;
 	else
 		return gfx_options + index;
@@ -890,19 +890,21 @@ write_error:
 	return FALSE;
 }
 
+static vbi_export_info
+info_ppm = {
+	.keyword	= "ppm",
+	.label		= N_("PPM"),
+	.tooltip	= N_("Export this page as raw PPM image"),
+
+	.mime_type	= "image/x-portable-pixmap",
+	.extension	= "ppm",
+};
+
 vbi_export_class
 vbi_export_class_ppm = {
-	.public = {
-		.keyword	= "ppm",
-		.label		= N_("PPM"),
-		.tooltip	= N_("Export this page as raw PPM image"),
-
-		.mime_type	= "image/x-portable-pixmap",
-		.extension	= "ppm",
-	},
-
-	.new			= gfx_new,
-	.delete			= gfx_delete,
+	._public		= &info_ppm,
+	._new			= gfx_new,
+	._delete		= gfx_delete,
 	.option_enum		= option_enum,
 	.option_get		= option_get,
 	.option_set		= option_set,
@@ -1161,21 +1163,21 @@ png_export(vbi_export *e, FILE *fp, vbi_page *pg)
 			title[0] = 0;
 
 		/*
+		 *  FIXME
 		 *  ISO 8859-1 (Latin-1) character set required,
 		 *  see png spec for other
 		 */
-		if (pg->pgno < 0x100)
+		if (pg->pgno < 0x100) {
 			size += snprintf(title + size, sizeof(title) - size - 1,
 					 "Closed Caption"); /* no i18n, proper name */
-		else if (pg->subno != VBI_ANY_SUBNO)
+		} else if (pg->subno != VBI_ANY_SUBNO) {
 			size += snprintf(title + size, sizeof(title) - size - 1,
-					 /* NLS: .png title, must be Latin-1 */
 					 _("Teletext Page %3x.%x"),
 					 pg->pgno, pg->subno);
-		else
+		} else {
 			size += snprintf(title + size, sizeof(title) - size - 1,
-					 /* NLS: .png title, must be Latin-1 */
 					 _("Teletext Page %3x"), pg->pgno);
+		}
 	}
 
 	memset(text, 0, sizeof(text));
@@ -1240,23 +1242,25 @@ unknown_error:
 	return FALSE;
 }
 
+static vbi_export_info
+info_png = {
+	.keyword	= "png",
+	.label		= N_("PNG"),
+	.tooltip	= N_("Export this page as PNG image"),
+
+	.mime_type	= "image/png",
+	.extension	= "png",
+};
+
 vbi_export_class
 vbi_export_class_png = {
-	.public = {
-		.keyword	= "png",
-		.label		= N_("PNG"),
-		.tooltip	= N_("Export this page as PNG image"),
-
-		.mime_type	= "image/png",
-		.extension	= "png",
-	},
-
-	.new			= gfx_new,
-	.delete			= gfx_delete,
-	.option_enum		= option_enum,
-	.option_get		= option_get,
-	.option_set		= option_set,
-	.export			= png_export
+	._public	= &info_png,
+	._new		= gfx_new,
+	._delete	= gfx_delete,
+	.option_enum	= option_enum,
+	.option_get	= option_get,
+	.option_set	= option_set,
+	.export		= png_export
 };
 
 VBI_AUTOREG_EXPORT_MODULE(vbi_export_class_png)
