@@ -17,13 +17,15 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-static char rcsid[] = "$Id: io-v4l.c,v 1.4 2002/03/19 19:27:40 mschimek Exp $";
+static char rcsid[] = "$Id: io-v4l.c,v 1.20 2003/11/14 05:29:18 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "../config.h"
 #endif
 
+#include "misc.h"
 #include "vbi.h"
+#include "io.h"
 
 #ifdef ENABLE_V4L
 
@@ -34,14 +36,13 @@ static char rcsid[] = "$Id: io-v4l.c,v 1.4 2002/03/19 19:27:40 mschimek Exp $";
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <getopt.h>
 #include <assert.h>
 #include <sys/time.h>		/* timeval */
 #include <sys/types.h>		/* fd_set */
 #include <sys/ioctl.h>
 #include <sys/mman.h>
+#include <pthread.h>
 
-#include "io.h"
 #include "videodev.h"
 
 /* same as ioctl(), but repeat if interrupted */
@@ -160,7 +161,7 @@ v4l_read(vbi_capture *vc, vbi_capture_buffer **raw,
 #include <sys/sysmacros.h>
 
 static void
-perm_check(char *name, vbi_bool trace)
+perm_check(const char *name, vbi_bool trace)
 {
 	struct stat st;
 	int old_errno = errno;
@@ -290,7 +291,7 @@ guess_bttv_v4l(vbi_capture_v4l *v, int *strict,
 	struct stat vbi_stat;
 	DIR *dir;
 	int mode = -1;
-	int i;
+	unsigned int i;
 
 	if (scanning) {
 		v->dec.scanning = scanning;
@@ -396,7 +397,7 @@ guess_bttv_v4l(vbi_capture_v4l *v, int *strict,
 
 static inline vbi_bool
 set_parameters(vbi_capture_v4l *v, struct vbi_format *vfmt, int *max_rate,
-	       char *dev_name, char *driver_name,
+	       const char *dev_name, char *driver_name,
 	       unsigned int *services, int strict,
 	       char **errorstr, vbi_bool trace)
 {
@@ -488,8 +489,16 @@ v4l_delete(vbi_capture *vc)
 	free(v);
 }
 
+static int
+v4l_fd(vbi_capture *vc)
+{
+	vbi_capture_v4l *v = PARENT(vc, vbi_capture_v4l, capture);
+
+	return v->fd;
+}
+
 static vbi_capture *
-v4l_new(char *dev_name, int given_fd, int scanning,
+v4l_new(const char *dev_name, int given_fd, int scanning,
 	unsigned int *services, int strict,
 	char **errorstr, vbi_bool trace)
 {
@@ -499,6 +508,8 @@ v4l_new(char *dev_name, int given_fd, int scanning,
 	char *driver_name = _("driver unknown");
 	vbi_capture_v4l *v;
 
+	pthread_once (&vbi_init_once, vbi_init);
+
 	assert(services && *services != 0);
 
 	if (scanning != 525 && scanning != 625)
@@ -507,14 +518,15 @@ v4l_new(char *dev_name, int given_fd, int scanning,
 	printv("Try to open v4l vbi device, libzvbi interface rev.\n"
 	       "%s", rcsid);
 
-	if (!(v = calloc(1, sizeof(*v)))) {
+	if (!(v = (vbi_capture_v4l *) calloc(1, sizeof(*v)))) {
 		vbi_asprintf(errorstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		return NULL;
 	}
 
 	v->capture.parameters = v4l_parameters;
-	v->capture.delete = v4l_delete;
+	v->capture._delete = v4l_delete;
+	v->capture.get_fd = v4l_fd;
 
 	if ((v->fd = open(dev_name, O_RDONLY)) == -1) {
 		vbi_asprintf(errorstr, _("Cannot open '%s': %d, %s."),
@@ -592,11 +604,11 @@ v4l_new(char *dev_name, int given_fd, int scanning,
 		v->dec.sampling_rate		= vfmt.sampling_rate;
 		v->dec.bytes_per_line 		= vfmt.samples_per_line;
 		if (v->dec.scanning == 625)
-			v->dec.offset 		= 10.2e-6 * vfmt.sampling_rate;
+			v->dec.offset 		= (int)(10.2e-6 * vfmt.sampling_rate);
 		else if (v->dec.scanning == 525)
-			v->dec.offset		= 9.2e-6 * vfmt.sampling_rate;
+			v->dec.offset		= (int)(9.2e-6 * vfmt.sampling_rate);
 		else /* we don't know */
-			v->dec.offset		= 9.7e-6 * vfmt.sampling_rate;
+			v->dec.offset		= (int)(9.7e-6 * vfmt.sampling_rate);
 		v->dec.start[0] 		= vfmt.start[0];
 		v->dec.count[0] 		= vfmt.count[0];
 		v->dec.start[1] 		= vfmt.start[1];
@@ -673,7 +685,7 @@ v4l_new(char *dev_name, int given_fd, int scanning,
 		case 625:
 			/* Not confirmed */
 			v->dec.sampling_rate = 35468950;
-			v->dec.offset = 10.2e-6 * 35468950;
+			v->dec.offset = (int)(10.2e-6 * 35468950);
 			v->dec.start[0] = 22 + 1 - v->dec.count[0];
 			v->dec.start[1] = 335 + 1 - v->dec.count[1];
 			break;
@@ -681,7 +693,7 @@ v4l_new(char *dev_name, int given_fd, int scanning,
 		case 525:
 			/* Confirmed for bttv 0.7.52 */
 			v->dec.sampling_rate = 28636363;
-			v->dec.offset = 9.2e-6 * 28636363;
+			v->dec.offset = (int)(9.2e-6 * 28636363);
 			v->dec.start[0] = 10;
 			v->dec.start[1] = 273;
 			break;
@@ -823,16 +835,16 @@ io_error:
 }
 
 vbi_capture *
-vbi_capture_v4l_sidecar_new(char *dev_name, int given_fd,
+vbi_capture_v4l_sidecar_new(const char *dev_name, int video_fd,
 			    unsigned int *services, int strict,
 			    char **errorstr, vbi_bool trace)
 {
-	return v4l_new(dev_name, given_fd, 0,
+	return v4l_new(dev_name, video_fd, 0,
 		       services, strict, errorstr, trace);
 }
 
 vbi_capture *
-vbi_capture_v4l_new(char *dev_name, int scanning,
+vbi_capture_v4l_new(const char *dev_name, int scanning,
 		    unsigned int *services, int strict,
 		    char **errorstr, vbi_bool trace)
 {
@@ -843,19 +855,21 @@ vbi_capture_v4l_new(char *dev_name, int scanning,
 #else
 
 vbi_capture *
-vbi_capture_v4l_sidecar_new(char *dev_name, int given_fd,
+vbi_capture_v4l_sidecar_new(const char *dev_name, int given_fd,
 			    unsigned int *services, int strict,
 			    char **errorstr, vbi_bool trace)
 {
+	pthread_once (&vbi_init_once, vbi_init);
 	vbi_asprintf(errorstr, _("V4L interface not compiled."));
 	return NULL;
 }
 
 vbi_capture *
-vbi_capture_v4l_new(char *dev_name, int scanning,
+vbi_capture_v4l_new(const char *dev_name, int scanning,
 		     unsigned int *services, int strict,
 		     char **errorstr, vbi_bool trace)
 {
+	pthread_once (&vbi_init_once, vbi_init);
 	vbi_asprintf(errorstr, _("V4L interface not compiled."));
 	return NULL;
 }
