@@ -17,13 +17,15 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-static char rcsid[] = "$Id: io-v4l2.c,v 1.4 2002/03/19 19:27:40 mschimek Exp $";
+static char rcsid[] = "$Id: io-v4l2.c,v 1.25 2004/02/18 07:56:45 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "../config.h"
 #endif
 
+#include "misc.h"
 #include "vbi.h"
+#include "io.h"
 
 #ifdef ENABLE_V4L2
 
@@ -34,15 +36,14 @@ static char rcsid[] = "$Id: io-v4l2.c,v 1.4 2002/03/19 19:27:40 mschimek Exp $";
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <getopt.h>
 #include <assert.h>
 #include <sys/time.h>		/* timeval */
 #include <sys/types.h>		/* fd_set */
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <asm/types.h>		/* for videodev2.h */
+#include <pthread.h>
 
-#include "io.h"
 #include "videodev2.h"
 
 /* same as ioctl(), but repeat if interrupted */
@@ -276,6 +277,14 @@ v4l2_delete(vbi_capture *vc)
 	free(v);
 }
 
+static int
+v4l2_fd(vbi_capture *vc)
+{
+	vbi_capture_v4l2 *v = PARENT(vc, vbi_capture_v4l2, capture);
+
+	return v->fd;
+}
+
 static void
 print_vfmt(char *s, struct v4l2_format *vfmt)
 {
@@ -288,29 +297,9 @@ print_vfmt(char *s, struct v4l2_format *vfmt)
 		vfmt->fmt.vbi.flags);
 }
 
-/**
- * vbi_capture_v4l2_new:
- * @dev_name: Name of the device to open, usually one of
- *   <filename>/dev/vbi</> or <filename>/dev/vbi0</> and up.
- * @buffers: Number of device buffers for raw vbi data, when
- *   the driver supports streaming. Otherwise one bounce buffer
- *   is allocated for vbi_capture_pull().
- * @services: This must point to a set of VBI_SLICED_ describing the
- *   data services to be decoded. On return the services actually
- *   decodable will be stored here. See #vbi_raw_decoder_add()
- *   for details. If you want to capture raw data only, set to
- *   #VBI_SLICED_VBI_525, #VBI_SLICED_VBI_625 or both.
- * @strict: Will be passed to #vbi_raw_decoder_add().
- * @errorstr: If not %NULL this function stores a pointer to an error
- *   description here. You must free() this string when no longer needed.
- * @trace: If %TRUE print progress messages on stderr.
- * 
- * 
- * 
- * Return value: 
- **/
+/* document below */
 vbi_capture *
-vbi_capture_v4l2_new(char *dev_name, int buffers,
+vbi_capture_v4l2_new(const char *dev_name, int buffers,
 		     unsigned int *services, int strict,
 		     char **errorstr, vbi_bool trace)
 {
@@ -323,6 +312,8 @@ vbi_capture_v4l2_new(char *dev_name, int buffers,
 	vbi_capture_v4l2 *v;
 	int max_rate, g_fmt;
 
+	pthread_once (&vbi_init_once, vbi_init);
+
 	assert(services && *services != 0);
 
 	printv("Try to open v4l2 vbi device, libzvbi interface rev.\n"
@@ -335,21 +326,33 @@ vbi_capture_v4l2_new(char *dev_name, int buffers,
 	}
 
 	v->capture.parameters = v4l2_parameters;
-	v->capture.delete = v4l2_delete;
+	v->capture._delete = v4l2_delete;
+	v->capture.get_fd = v4l2_fd;
 
-	if ((v->fd = open(dev_name, O_RDONLY)) == -1) {
-		vbi_asprintf(errorstr, _("Cannot open '%s': %d, %s."),
-			     dev_name, errno, strerror(errno));
-		goto io_error;
+	/* O_RDWR required for PROT_WRITE */
+	if ((v->fd = open(dev_name, O_RDWR)) == -1) {
+		if ((v->fd = open(dev_name, O_RDONLY)) == -1) {
+			vbi_asprintf(errorstr, _("Cannot open '%s': %d, %s."),
+				     dev_name, errno, strerror(errno));
+			goto io_error;
+		}
 	}
 
 	printv("Opened %s\n", dev_name);
 
 	if (IOCTL(v->fd, VIDIOC_QUERYCAP, &vcap) == -1) {
+/*
 		vbi_asprintf(errorstr, _("Cannot identify '%s': %d, %s."),
 			     dev_name, errno, strerror(errno));
 		guess = _("Probably not a v4l2 device.");
+
 		goto io_error;
+*/
+		v4l2_delete (&v->capture);
+
+		/* Try api revision 2002-10 */
+		return vbi_capture_v4l2k_new (dev_name, 0, buffers,
+					      services, strict, errorstr, trace);
 	}
 
 	if (vcap.type != V4L2_TYPE_VBI) {
@@ -370,8 +373,8 @@ vbi_capture_v4l2_new(char *dev_name, int buffers,
 	}
 #endif
 
+	/* mandatory, http://www.thedirks.org/v4l2/v4l2dsi.htm */
 	if (IOCTL(v->fd, VIDIOC_G_STD, &vstd) == -1) {
-		/* mandatory, http://www.thedirks.org/v4l2/v4l2dsi.htm */
 		vbi_asprintf(errorstr, _("Cannot query current videostandard of %s (%s): %d, %s."),
 			     dev_name, vcap.name, errno, strerror(errno));
 		guess = _("Probably a driver bug.");
@@ -505,9 +508,16 @@ vbi_capture_v4l2_new(char *dev_name, int buffers,
 	v->dec.sampling_format = VBI_PIXFMT_YUV420;
 
 	if (*services & ~(VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625)) {
-		/* Nyquist */
+		/* Nyquist (we're generous at 1.5) */
 
 		if (v->dec.sampling_rate < max_rate * 3 / 2) {
+			vbi_asprintf(errorstr, _("Cannot capture the requested "
+						 "data services with "
+						 "%s (%s), the sampling frequency "
+						 "%.2f MHz is too low."),
+				     dev_name, vcap.name,
+				     v->dec.sampling_rate / 1e6);
+			goto failure;
 		}
 
 		printv("Nyquist check passed\n");
@@ -617,7 +627,7 @@ vbi_capture_v4l2_new(char *dev_name, int buffers,
 					     errno, strerror(errno));
 				goto mmap_failure;
 			} else {
-				int i, s;
+				unsigned int i, s;
 
 				v->raw_buffer[v->num_raw_buffers].data = p;
 				v->raw_buffer[v->num_raw_buffers].size = vbuf.length;
@@ -702,11 +712,32 @@ failure:
 
 #else
 
+/**
+ * @param dev_name Name of the device to open, usually one of
+ *   @c /dev/vbi or @c /dev/vbi0 and up.
+ * @param buffers Number of device buffers for raw vbi data, when
+ *   the driver supports streaming. Otherwise one bounce buffer
+ *   is allocated for vbi_capture_pull().
+ * @param services This must point to a set of @ref VBI_SLICED_
+ *   symbols describing the
+ *   data services to be decoded. On return the services actually
+ *   decodable will be stored here. See vbi_raw_decoder_add()
+ *   for details. If you want to capture raw data only, set to
+ *   @c VBI_SLICED_VBI_525, @c VBI_SLICED_VBI_625 or both.
+ * @param strict Will be passed to vbi_raw_decoder_add().
+ * @param errorstr If not @c NULL this function stores a pointer to an error
+ *   description here. You must free() this string when no longer needed.
+ * @param trace If @c TRUE print progress messages on stderr.
+ * 
+ * @return
+ * Initialized vbi_capture context, @c NULL on failure.
+ */
 vbi_capture *
-vbi_capture_v4l2_new(char *dev_name, int buffers,
+vbi_capture_v4l2_new(const char *dev_name, int buffers,
 		     unsigned int *services, int strict,
 		     char **errorstr, vbi_bool trace)
 {
+	pthread_once (&vbi_init_once, vbi_init);
 	vbi_asprintf(errorstr, _("V4L2 interface not compiled."));
 	return NULL;
 }
