@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lang.c,v 1.2 2002/03/19 19:27:40 mschimek Exp $ */
+/* $Id: lang.c,v 1.4 2002/10/11 12:31:49 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "../config.h"
@@ -44,7 +44,7 @@ vbi_font_descriptors[88] = {
 	{ LATIN_G0, LATIN_G2, FRENCH,		"Franais" },
 	{ LATIN_G0, LATIN_G2, PORTUG_SPANISH,	"Portugus / Espaol" },
 	{ LATIN_G0, LATIN_G2, CZECH_SLOVAK,	"Cesky / Slovencina" },
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
 
 	/* 8 - Eastern Europe */
 	{ LATIN_G0, LATIN_G2, POLISH,		"Polski" },
@@ -52,9 +52,9 @@ vbi_font_descriptors[88] = {
 	{ LATIN_G0, LATIN_G2, SWE_FIN_HUN,	"Svenska / Suomi / Magyar" },
 	{ LATIN_G0, LATIN_G2, ITALIAN,		"Italiano" },
 	{ LATIN_G0, LATIN_G2, FRENCH,		"Franais" },
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
 	{ LATIN_G0, LATIN_G2, CZECH_SLOVAK,	"Cesky / Slovencina" },
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
 
 	/* 16 - Western Europe and Turkey */
 	{ LATIN_G0, LATIN_G2, ENGLISH,		"English" },
@@ -64,16 +64,16 @@ vbi_font_descriptors[88] = {
 	{ LATIN_G0, LATIN_G2, FRENCH,		"Franais" },
 	{ LATIN_G0, LATIN_G2, PORTUG_SPANISH,	"Portugus / Espaol" },
 	{ LATIN_G0, LATIN_G2, TURKISH,		"Trke" },
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
 
 	/* 24 - Central and Southeast Europe */
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
 	{ LATIN_G0, LATIN_G2, SERB_CRO_SLO,	"Srbski / Hrvatski / Slovenscina" },
-	{ LATIN_G0, LATIN_G2, NO_SUBSET		},
+	{ LATIN_G0, LATIN_G2, NO_SUBSET,	0 },
 	{ LATIN_G0, LATIN_G2, RUMANIAN,		"Romna" },
 
 	/* 32 - Cyrillic */
@@ -84,63 +84,63 @@ vbi_font_descriptors[88] = {
 	{ CYRILLIC_2_G0, CYRILLIC_G2, NO_SUBSET, "Russky / Balgarski " },
 	{ CYRILLIC_3_G0, CYRILLIC_G2, NO_SUBSET, "Ukrayins'ka" },
 	{ LATIN_G0, LATIN_G2, CZECH_SLOVAK,	"Cesky / Slovencina" },
-	{ 0, 0, NO_SUBSET,			},
+	{ 0, 0, NO_SUBSET,			0 },
 
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 
 	/* 48 */
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 	{ LATIN_G0, LATIN_G2, TURKISH,		"Trke" },
 	{ GREEK_G0, GREEK_G2, NO_SUBSET,	"Ellinika'" },
 
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 
 	/* 64 - Arabic */
 	{ LATIN_G0, ARABIC_G2, ENGLISH,		"Alarabia / English" },
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 	{ LATIN_G0, ARABIC_G2, FRENCH,		"Alarabia / Franais" },
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 	{ ARABIC_G0, ARABIC_G2, NO_SUBSET,	"Alarabia" },
 
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 
 	/* 80 */
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
+	{ 0, 0, NO_SUBSET,			0 },
 	{ HEBREW_G0, ARABIC_G2, NO_SUBSET,	"Ivrit" },
-	{ 0, 0, NO_SUBSET			},
+	{ 0, 0, NO_SUBSET,			0 },
 	{ ARABIC_G0, ARABIC_G2, NO_SUBSET,	"Alarabia" },
 };
 
@@ -349,11 +349,11 @@ hebrew_g0[37] = {
 };
 
 /**
- * vbi_teletext_unicode:
- * @s: Teletext character set as listed in ETS 300 706 section 15.
- * @n: National character subset as listed in section 15, only
+ * @internal
+ * @param s Teletext character set as listed in ETS 300 706 section 15.
+ * @param n National character subset as listed in section 15, only
  *     applicable to character set LATIN_G0, ignored otherwise.
- * @c: Character code in range 0x20 ... 0x7F.
+ * @param c Character code in range 0x20 ... 0x7F.
  * 
  * Translate Teletext character code to Unicode.
  * 
@@ -379,9 +379,9 @@ hebrew_g0[37] = {
  * 
  * Private codes U+F000 ... U+F7FF are reserved for DRCS.
  * 
- * Return value:
+ * @return
  * Unicode value.
- **/
+ */
 unsigned int
 vbi_teletext_unicode(vbi_character_set s, vbi_national_subset n, unsigned int c)
 {
@@ -503,21 +503,21 @@ composed[12 * 16] = {
 };
 
 /**
- * vbi_teletext_composed_unicode:
- * @a: Accent 0 ... 15.
- * @c: Character code in range 0x20 ... 0x7F.
+ * @internal
+ * @param a Accent 0 ... 15.
+ * @param c Character code in range 0x20 ... 0x7F.
  * 
  * Translate Teletext Latin G1 character 0x20 ... 0x7F combined
  * with accent code from Latin G2 0x40 ... 0x4F to Unicode. Not all
  * combinations are representable in Unicode.
  *
- * Return value:
+ * @return
  * Unicode value or 0.
- **/
+ */
 unsigned int
 vbi_teletext_composed_unicode(unsigned int a, unsigned int c)
 {
-	int i;
+	unsigned int i;
 
 	assert(a <= 15);
 	assert(c >= 0x20 && c <= 0x7F);
@@ -538,7 +538,7 @@ vbi_teletext_composed_unicode(unsigned int a, unsigned int c)
  *
  *  EIA 608
  *
- *  (from Video Demystifiedu, Table 8.31. Closed Captioning Basic Character Set)
+ *  (from Video Demystified, Table 8.31. Closed Captioning Basic Character Set)
  */
 static const unsigned short
 caption[96] = {
@@ -557,16 +557,16 @@ caption_special[] = {
 };
 
 /**
- * vbi_caption_unicode:
- * @c: Character code in range 0x00 ... 0x0F and 0x20 ... 0x7F.
+ * @internal
+ * @param c Character code in range 0x00 ... 0x0F and 0x20 ... 0x7F.
  * 
  * Translate Closed Caption character code to Unicode. Codes
  * in range 0x00 ... 0x0F are Special Characters (Closed
  * Caption commands 0x22 0x3x, 0x32 0x3x).
  *
- * Return value:
+ * @return
  * Unicode value.
- **/
+ */
 unsigned int
 vbi_caption_unicode(unsigned int c)
 {
@@ -603,15 +603,15 @@ isfull(vbi_char c)
 }
 
 /**
- * vbi_optimize_page:
- * @pg: Formatted #vbi_page to be optimized.
- * @column: First column, 0 ... pg->columns - 1.
- * @row: First row, 0 ... pg->rows - 1.
- * @width: Number of columns to optimize, 1 ... pg->columns.
- * @height: Number of rows to optimize, 1 ... pg->rows.
+ * @internal
+ * @param pg Formatted vbi_page to be optimized.
+ * @param column First column, 0 ... pg->columns - 1.
+ * @param row First row, 0 ... pg->rows - 1.
+ * @param width Number of columns to optimize, 1 ... pg->columns.
+ * @param height Number of rows to optimize, 1 ... pg->rows.
  *
  * Experimental.
- **/
+ */
 void
 vbi_optimize_page(vbi_page *pg, int column, int row, int width, int height)
 {
