@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: packet.c,v 1.1.1.1 2002/01/12 16:18:52 mschimek Exp $ */
+/* $Id: packet.c,v 1.13 2003/11/14 05:29:18 mschimek Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,12 +29,17 @@
 #include <errno.h>
 #include <sys/ioctl.h>
 
+#include "misc.h"
 #include "hamm.h"
 #include "lang.h"
 #include "export.h"
 #include "tables.h"
 #include "vbi.h"
 
+#ifndef FPC
+#define FPC 0
+#endif
+
 #define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? '.' : ((c) & 0x7F))
 
 static vbi_bool convert_drcs(vt_page *vtp, uint8_t *raw);
@@ -66,7 +71,7 @@ dump_raw(vt_page *vtp, vbi_bool unham)
 }
 
 static void
-dump_extension(extension *ext)
+dump_extension(vt_extension *ext)
 {
 	int i;
 
@@ -158,7 +163,7 @@ hamm8_page_number(pagenum *p, uint8_t *raw, int magazine)
 }
 
 static inline vbi_bool
-parse_mot(magazine *mag, uint8_t *raw, int packet)
+parse_mot(vt_magazine *mag, uint8_t *raw, int packet)
 {
 	int err, i, j;
 
@@ -166,7 +171,7 @@ parse_mot(magazine *mag, uint8_t *raw, int packet)
 	case 1 ... 8:
 	{
 		int index = (packet - 1) << 5;
-		char n0, n1;
+		int n0, n1;
 
 		for (i = 0; i < 20; index++, i++) {
 			if (i == 10)
@@ -190,7 +195,7 @@ parse_mot(magazine *mag, uint8_t *raw, int packet)
 		int index = (packet - 9) * 0x30 + 10;
 
 		for (i = 0; i < 20; index++, i++) {
-			char n0, n1;
+			int n0, n1;
 
 			if (i == 6 || i == 12) {
 				if (index == 0x100)
@@ -220,10 +225,10 @@ parse_mot(magazine *mag, uint8_t *raw, int packet)
 
 	case 19 ... 20: /* level 2.5 pops */
 	{
-		pop_link *pop = mag->pop_link + (packet - 19) * 4;
+		vt_pop_link *pop = mag->pop_link + (packet - 19) * 4;
 
 		for (i = 0; i < 4; raw += 10, pop++, i++) {
-			char n[10];
+			int n[10];
 
 			for (err = j = 0; j < 10; j++)
 				err |= n[j] = vbi_hamm8(raw[j]);
@@ -259,7 +264,7 @@ parse_mot(magazine *mag, uint8_t *raw, int packet)
 	case 24:	/* level 3.5 drcs */
 	    {
 		int index = (packet == 21) ? 0 : 8;
-		char n[4];
+		int n[4];
 
 		for (i = 0; i < 8; raw += 4, index++, i++) {
 			for (err = j = 0; j < 4; j++)
@@ -454,8 +459,8 @@ convert_drcs(vt_page *vtp, uint8_t *raw)
 static int
 page_language(struct teletext *vt, vt_page *vtp, int pgno, int national)
 {
-	magazine *mag;
-	extension *ext;
+	vt_magazine *mag;
+	vt_extension *ext;
 	int char_set;
 	int lang = -1; /***/
 
@@ -655,7 +660,7 @@ static const int dec2bcdp[20] = {
 static vbi_bool
 top_page_number(pagenum *p, uint8_t *raw)
 {
-	char n[8];
+	int n[8];
 	int pgno, err, i;
 
 	for (err = i = 0; i < 8; i++)
@@ -874,18 +879,18 @@ parse_mpt_ex(struct teletext *vt, uint8_t *raw, int packet)
 }
 
 /**
- * vbi_convert_page:
- * @vbi: Initialized vbi decoding context.
- * @vtp: Raw teletext page to be converted.
- * @cached: The raw page is already cached, update the cache.
- * @new_function: The page function to convert to.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
+ * @param vtp Raw teletext page to be converted.
+ * @param cached The raw page is already cached, update the cache.
+ * @param new_function The page function to convert to.
  * 
  * Since MOT, MIP and X/28 are optional, the function of a system page
  * may not be clear until we format a LOP and find a link of certain type,
  * so this function converts a page "after the fact".
  * 
- * Return value: 
- * Pointer to the converted page, either @vtp or the cached copy.
+ * @return
+ * Pointer to the converted page, either @a vtp or the cached copy.
  **/
 vt_page *
 vbi_convert_page(vbi_decoder *vbi, vt_page *vtp,
@@ -987,7 +992,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 	case CNI_8301:
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni1 == cni) {
-				*country = vbi_country_names_en[p->country];
+				*country = p->country;
 				*name = p->name;
 				return p->id;
 			}
@@ -996,7 +1001,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 	case CNI_8302:
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni2 == cni) {
-				*country = vbi_country_names_en[p->country];
+				*country = p->country;
 				*name = p->name;
 				return p->id;
 			}
@@ -1011,7 +1016,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni4 == cni) {
-				*country = vbi_country_names_en[p->country];
+				*country = p->country;
 				*name = p->name;
 				return p->id;
 			}
@@ -1020,7 +1025,7 @@ station_lookup(vbi_cni_type type, int cni, const char **country, const char **na
 	case CNI_X26:
 		for (p = vbi_cni_table; p->name; p++)
 			if (p->cni3 == cni) {
-				*country = vbi_country_names_en[p->country];
+				*country = p->country;
 				*name = p->name;
 				return p->id;
 			}
@@ -1094,14 +1099,17 @@ dump_pil(int pil)
 static void
 dump_pty(int pty)
 {
+	extern const char *ets_program_class[16];
+	extern const char *ets_program_type[8][16];
+
 	if (pty == 0xFF)
 		printf("... prog. type: %02x unused", pty);
 	else
-		printf("... prog. type: %02x class %s", pty, program_class[pty >> 4]);
+		printf("... prog. type: %02x class %s", pty, ets_program_class[pty >> 4]);
 
 	if (pty < 0x80) {
-		if (program_type[pty >> 4][pty & 0xF])
-			printf(", type %s", program_type[pty >> 4][pty & 0xF]);
+		if (ets_program_type[pty >> 4][pty & 0xF])
+			printf(", type %s", ets_program_type[pty >> 4][pty & 0xF]);
 		else
 			printf(", type undefined");
 	}
@@ -1112,14 +1120,14 @@ dump_pty(int pty)
 #endif /* BSDATA_TEST */
 
 /**
- * vbi_decode_vps:
- * @vbi: Initialized vbi decoding context.
- * @buf: 13 bytes.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
+ * @param buf 13 bytes.
  * 
  * Decode a VPS datagram (13 bytes) according to
  * ETS 300 231 and update decoder state. This may
- * send a VBI_EVENT_NETWORK.
- **/
+ * send a @a VBI_EVENT_NETWORK.
+ */
 void
 vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 {
@@ -1146,6 +1154,7 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 			unknown_cni(vbi, "VPS", cni);
 		} else {
 			strncpy(n->name, name, sizeof(n->name) - 1);
+			n->name[sizeof(n->name) - 1] = 0;
 		}
 
 		if (id != n->nuid) {
@@ -1161,9 +1170,7 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 		n->cycle = 2;
 	}
 
-#if BSDATA_TEST
-
-	{
+	if (BSDATA_TEST && 0) {
 		static char pr_label[20];
 		static char label[20];
 		static int l = 0;
@@ -1172,9 +1179,9 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 
 		printf("\nVPS:\n");
 
-		c = bit_reverse[buf[1]];
+		c = vbi_bit_reverse[buf[1]];
 
-		if ((char) c < 0) {
+		if ((int8_t) c < 0) {
 			label[l] = 0;
 			memcpy(pr_label, label, sizeof(pr_label));
 			l = 0;
@@ -1195,11 +1202,11 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 		      + (buf[11] & 0x3F);
 
 		if (cni)
-			for (j = 0; VPS_CNI[j].short_name; j++)
-				if (VPS_CNI[j].cni == cni) {
+			for (j = 0; vbi_cni_table[j].name; j++)
+				if (vbi_cni_table[j].cni4 == cni) {
 					printf(" Country: %s\n Station: %s%s\n",
-						country_names_en[VPS_CNI[j].country],
-						VPS_CNI[j].long_name,
+						vbi_cni_table[j].country,
+						vbi_cni_table[j].name,
 						(cni == 0x0DC3) ? ((buf[2] & 0x10) ? " (ZDF)" : " (ARD)") : "");
 					break;
 				}
@@ -1208,96 +1215,15 @@ vbi_decode_vps(vbi_decoder *vbi, uint8_t *buf)
 		pil = ((buf[8] & 0x3F) << 14) + (buf[9] << 6) + (buf[10] >> 2);
 		pty = buf[12];
 
-/*		if (!cni || !VPS_CNI[j].short_name) */
+		/* if (!cni || !vbi_cni_table[j].name) */
 			printf(" CNI: %04x\n", cni);
-
+#if BSDATA_TEST
 		printf(" Analog audio: %s\n", pcs_names[pcs]);
 
 		dump_pil(pil);
 		dump_pty(pty);
-	}
-
-#endif /* BSDATA_TEST */
-
-}
-
-static void
-parse_x26_pdc(int address, int mode, int data)
-{
-	static int day, month, lto, caf, duration;
-	static int hour[2], min[2], mi = 0;
-	int cni;
-
-	if (mode != 6 && address < 40)
-		return;
-
-	switch (mode) {
-	case 6:
-		if (address >= 40)
-			return;
-		min[mi] = data & 0x7F; // BCD
-		break;
-
-	case 8:
-		cni = address * 256 + data;
-
-		if (0) { /* country and network identifier */
-			const char *country, *name;
-
-			if (station_lookup(CNI_X26, cni, &country, &name))
-				printf("X/26 country: %s\n... station: %s\n", country, name);
-			else
-				printf("X/26 unknown CNI %04x\n", cni);
-		}
-
-		return;
-
-	case 9:
-		month = address & 15;
-		day = (data >> 4) * 10 + (data & 15);
-		break;
-
-	case 10:
-		hour[0] = data & 0x3F; // BCD
-		caf = !!(data & 0x40);
-		mi = 0;
-		break;
-
-	case 11:
-		hour[1] = data & 0x3F; // BCD
-		duration = !!(data & 0x40);
-		mi = 1;
-		break;
-
-	case 12:
-		lto = (data & 0x40) ? ((~0x7F) | data) : data;
-		break;
-
-	case 13:
-#if BSDATA_TEST
-		if (0) {
-			printf("X/26 pty series: %d\n", address == 0x30);
-			dump_pty(data | 0x80);
-		}
 #endif
-		break;
-
-	default:
-		return;
 	}
-
-#if BSDATA_TEST
-
-	/*
-	 *  It's life, but not as we know it...
-	 */
-	if (0 && mode == 6)
-		printf("X/26 %2d/%2d/%2d; lto=%d, caf=%d, end/dur=%d; %d %s %02x:%02x %02x:%02x\n",
-			address, mode, data,
-			lto, caf, duration,
-			day, month_names[month],
-			hour[0], min[0], hour[1], min[1]);
-#endif
 }
 
 static vbi_bool
@@ -1337,6 +1263,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 					unknown_cni(vbi, "8/30/1", cni);
 				} else {
 					strncpy(n->name, name, sizeof(n->name) - 1);
+					n->name[sizeof(n->name) - 1] = 0;
 				}
 
 				if (id != n->nuid) {
@@ -1353,8 +1280,8 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			}
 #if BSDATA_TEST
 			if (1) { /* country and network identifier */
-				if (station_lookup(CNI_8301, cni, &country, &short_name, &long_name))
-					printf("... country: %s\n... station: %s\n", country, long_name);
+				if (station_lookup(CNI_8301, cni, &country, &name))
+					printf("... country: %s\n... station: %s\n", country, name);
 				else
 					printf("... unknown CNI %04x\n", cni);
 			}
@@ -1422,6 +1349,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 					unknown_cni(vbi, "8/30/2", cni);
 				} else {
 					strncpy(n->name, name, sizeof(n->name) - 1);
+					n->name[sizeof(n->name) - 1] = 0;
 				}
 
 				if (id != n->nuid) {
@@ -1442,7 +1370,7 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 				const char *country, *name;
 
 				if (station_lookup(CNI_8302, cni, &country, &name))
-					printf("... country: %s\n... station: %s\n", country, long_name);
+					printf("... country: %s\n... station: %s\n", country, name);
 				else
 					printf("... unknown CNI %04x\n", cni);
 			}
@@ -1450,11 +1378,11 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			if (1) { /* PDC data */
 				int lci, luf, prf, mi, pil;
 
-				lci = (n[0] >> 2) & 3;
-				luf = !!(n[0] & 2);
-				prf = n[0] & 1;
-				mi = !!(n[1] & 0x20);
-				pil = ((n[2] & 0x3F) << 14) + (n[3] << 6) + (n[4] >> 2);
+				lci = (b[0] >> 2) & 3;
+				luf = !!(b[0] & 2);
+				prf = b[0] & 1;
+				mi = !!(b[1] & 0x20);
+				pil = ((b[2] & 0x3F) << 14) + (b[3] << 6) + (b[4] >> 2);
 
 				printf("... label channel %d: update %d,"
 				       " prepare to record %d, mode %d\n",
@@ -1465,8 +1393,8 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 			if (1) {
 				int pty, pcs;
 
-				pcs = n[1] >> 6;
-				pty = n[6];
+				pcs = b[1] >> 6;
+				pty = b[6];
 
 				printf("... analog audio: %s\n", pcs_names[pcs]);
 				dump_pty(pty);
@@ -1499,6 +1427,103 @@ parse_bsd(vbi_decoder *vbi, uint8_t *raw, int packet, int designation)
 	return TRUE;
 }
 
+#define FPC_BLOCK_SEPARATOR	0xC
+#define FPC_FILLER_BYTE		0x3
+
+static inline void
+vbi_reset_page_clear(struct page_clear *pc)
+{
+	pc->ci = 256;
+	pc->packet = 256;
+	pc->num_packets = 0;
+	pc->bi = 0;
+	pc->left = 0;
+	pc->pfc.application_id = -1;
+}
+
+static void
+parse_page_clear(struct page_clear *pc, uint8_t *p, int packet)
+{
+	int bp, col;
+
+	if ((pc->packet + 1) != packet || packet > pc->num_packets)
+		goto desync;
+
+	pc->packet = packet;
+
+	if ((bp = vbi_hamm8(p[0]) * 3) < 0 || bp > 39)
+		goto desync;
+
+	for (col = 1; col < 40;) {
+		int bs;
+
+		if (pc->left > 0) {
+			int size = MIN(pc->left, 40 - col);
+
+			memcpy(pc->pfc.block + pc->bi, p + col, size);
+
+			pc->bi += size;
+			pc->left -= size;
+
+			if (pc->left > 0)
+				return; /* packet done */
+
+			col += size;
+
+			if (pc->pfc.application_id < 0) {
+				int sh = vbi_hamm16(pc->pfc.block)
+					+ vbi_hamm16(pc->pfc.block + 2) * 256;
+
+				pc->pfc.application_id = sh & 0x1F;
+				pc->pfc.block_size =
+					pc->left = sh >> 5;
+				pc->bi = 0;
+
+				continue;
+			} else {
+				int i;
+					
+				fprintf(stderr, "pfc %d %d\n",
+					pc->pfc.application_id,
+					pc->pfc.block_size);
+
+				for (i = 0; i < pc->pfc.block_size; i++) {
+					fputc(printable(pc->pfc.block[i]), stderr);
+
+					if ((i % 75) == 75)
+						fputc('\n', stderr);
+				}
+
+				fputc('\n', stderr);
+			}
+		}
+
+		if (col <= 1) {
+			if (bp >= 39)
+				return; /* no SH in this packet */
+			col = bp + 2;
+			bs = vbi_hamm8(p[col - 1]);
+		} else
+			while ((bs = vbi_hamm8(p[col++])) == FPC_FILLER_BYTE) {
+				if (col >= 40)
+					return; /* packet done */
+			}
+
+		if (bs != FPC_BLOCK_SEPARATOR)
+			goto desync;
+
+		pc->pfc.application_id = -1;
+		pc->left = 4; /* sizeof structure header */
+		pc->bi = 0;
+	}
+
+	return;
+
+ desync:
+	// fprintf(stderr, "FPC reset\n");
+	vbi_reset_page_clear(pc);
+}
+
 static int
 same_header(int cur_pgno, uint8_t *cur,
 	    int ref_pgno, uint8_t *ref,
@@ -1811,7 +1836,7 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 {
 	int designation, function;
 	int triplets[13], *triplet = triplets, buf = 0, left = 0;
-	extension *ext;
+	vt_extension *ext;
 	int i, j, err = 0;
 
 	static int
@@ -1836,7 +1861,9 @@ parse_28_29(vbi_decoder *vbi, uint8_t *p,
 	if ((designation = vbi_hamm8(*p)) < 0)
 		return FALSE;
 
-//	printf("Packet %d/%d/%d page %x\n", mag8, packet, designation, cvtp->pgno);
+	if (0)
+		fprintf(stderr, "Packet %d/%d/%d page %x\n",
+			mag8, packet, designation, cvtp->pgno);
 
 	for (p++, i = 0; i < 13; p += 3, i++)
 		err |= triplet[i] = vbi_hamm24(p);
@@ -2054,23 +2081,23 @@ parse_8_30(vbi_decoder *vbi, uint8_t *p, int packet)
 }
 
 /**
- * vbi_decode_teletext:
- * @vbi: Initialized vbi decoding context.
- * @p: Packet data.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
+ * @param p Packet data.
  * 
  * Parse a teletext packet (42 bytes) and update the decoder
  * state accordingly. This function may send events.
  * 
  * Return value:
  * FALSE if the packet contained incorrectable errors. 
- **/
+ */
 vbi_bool
 vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 {
 	vt_page *cvtp;
 	struct raw_page *rvtp;
 	int pmag, mag0, mag8, packet;
-	magazine *mag;
+	vt_magazine *mag;
 
 	if ((pmag = vbi_hamm16(p)) < 0)
 		return FALSE;
@@ -2089,6 +2116,15 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 
 	p += 2;
 
+	if (0) {
+		unsigned int i;
+
+		fprintf(stderr, "packet 0x%x %d >", mag8 * 0x100, packet);
+		for (i = 0; i < 40; i++)
+			fputc(printable(p[i]), stderr);
+		fprintf(stderr, "<\n");
+	}
+
 	switch (packet) {
 	case 0:
 	{
@@ -2124,6 +2160,7 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 
 			switch (vtp->function) {
 			case PAGE_FUNCTION_DISCARD:
+			case PAGE_FUNCTION_EPG:
 				break;
 
 			case PAGE_FUNCTION_LOP:
@@ -2174,7 +2211,8 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 		cvtp->flags = (flags << 16) + subpage;
 
 		if (0 && ((page & 15) > 9 || page > 0x99))
-			printf("data page %03x/%04x\n", cvtp->pgno, cvtp->subno);
+			printf("data page %03x/%04x n%d\n",
+			       cvtp->pgno, cvtp->subno, cvtp->national);
 
 		if (1
 		    && pgno != 0x1E7
@@ -2202,6 +2240,8 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 			cvtp->lop_lines = vtp->lop_lines;
 			cvtp->enh_lines = vtp->enh_lines;
 		} else {
+			struct page_info *pi = vbi->vt.page_info + cvtp->pgno - 0x100;
+
 			cvtp->flags |= C4_ERASE_PAGE;
 
 			if (0)
@@ -2209,20 +2249,41 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 
 			if (cvtp->pgno == 0x1F0) {
 				cvtp->function = PAGE_FUNCTION_BTT;
-				vbi->vt.page_info[0x1F0 - 0x100].code = VBI_TOP_PAGE;
+				pi->code = VBI_TOP_PAGE;
 			} else if (cvtp->pgno == 0x1E7) {
 				cvtp->function = PAGE_FUNCTION_TRIGGER;
-				vbi->vt.page_info[0x1E7 - 0x100].code = VBI_DISP_SYSTEM_PAGE;
-				vbi->vt.page_info[0x1E7 - 0x100].subcode = 0;
+				pi->code = VBI_DISP_SYSTEM_PAGE;
+				pi->subcode = 0;
 				memset(cvtp->data.unknown.raw[0], 0x20, sizeof(cvtp->data.unknown.raw));
 				memset(cvtp->data.enh_lop.enh, 0xFF, sizeof(cvtp->data.enh_lop.enh));
 				cvtp->data.unknown.ext = FALSE;
 			} else if (page == 0xFD) {
 				cvtp->function = PAGE_FUNCTION_MIP;
-				vbi->vt.page_info[cvtp->pgno - 0x100].code = VBI_SYSTEM_PAGE;
+				pi->code = VBI_SYSTEM_PAGE;
 			} else if (page == 0xFE) {
 				cvtp->function = PAGE_FUNCTION_MOT;
-				vbi->vt.page_info[cvtp->pgno - 0x100].code = VBI_SYSTEM_PAGE;
+				pi->code = VBI_SYSTEM_PAGE;
+			} else if (FPC && pi->code == VBI_EPG_DATA) {
+				int stream = (cvtp->subno >> 8) & 15;
+
+				if (stream >= 2) {
+					cvtp->function = PAGE_FUNCTION_DISCARD;
+					// fprintf(stderr, "Discard FPC %d\n", stream);
+				} else {
+					struct page_clear *pc = vbi->epg_pc + stream;
+					int ci = cvtp->subno & 15;
+
+					cvtp->function = PAGE_FUNCTION_EPG;
+					pc->pfc.pgno = cvtp->pgno;
+
+					if (((pc->ci + 1) & 15) != ci)
+						vbi_reset_page_clear(pc);
+
+					pc->ci = ci;
+					pc->packet = 0;
+					pc->num_packets = ((cvtp->subno >> 4) & 7)
+						+ ((cvtp->subno >> 9) & 0x18);
+				}
 			} else {
 				cvtp->function = PAGE_FUNCTION_UNKNOWN;
 
@@ -2295,8 +2356,15 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 				function = PAGE_FUNCTION_TRIGGER;
 				break;
 
-			case 0x52 ... 0x6F:	/* reserved */
 			case VBI_EPG_DATA:	/* EPG/NexTView transport layer */
+				if (FPC) {
+					function = PAGE_FUNCTION_EPG;
+					break;
+				}
+
+				/* fall through */
+
+			case 0x52 ... 0x6F:	/* reserved */
 			case VBI_ACI:		/* ACI page */
 			case VBI_NOT_PUBLIC:
 			case 0xD2 ... 0xDF:	/* reserved */
@@ -2326,7 +2394,7 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 
 	case 1 ... 25:
 	{
-		char n;
+		int n;
 		int i;
 
 		switch (cvtp->function) {
@@ -2369,6 +2437,10 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 				return FALSE;
 			break;
 
+		case PAGE_FUNCTION_EPG:
+			parse_page_clear(vbi->epg_pc + ((cvtp->subno >> 8) & 1), p, packet);
+			break;
+
 		case PAGE_FUNCTION_LOP:
 		case PAGE_FUNCTION_TRIGGER:
 			for (n = i = 0; i < 40; i++)
@@ -2442,12 +2514,6 @@ vbi_decode_teletext(vbi_decoder *vbi, uint8_t *p)
 			triplet.data = t >> 11;
 
 			cvtp->data.enh_lop.enh[rvtp->num_triplets++] = triplet;
-
-			if (0
-			    && triplet.mode >= 6
-			    && triplet.mode <= 13)
-				parse_x26_pdc(triplet.address,
-					      triplet.mode, triplet.data);
 		}
 
 		cvtp->enh_lines |= 1 << designation;
@@ -2525,9 +2591,8 @@ default_color_map[40] = {
 };
 
 /**
- * vbi_set_default_region:
- * @vbi: Initialized vbi decoding context.
- * @default_region: A value between 0 ... 80, index into
+ * @param vbi Initialized vbi decoding context.
+ * @param default_region A value between 0 ... 80, index into
  *   the Teletext character set table according to ETS 300 706,
  *   Section 15 (or libzvbi source file lang.c). The three last
  *   significant bits will be replaced.
@@ -2536,11 +2601,11 @@ default_color_map[40] = {
  * eight national character sets. When more countries started
  * to broadcast Teletext the three bit character set id was
  * locally redefined and later extended to seven bits grouping
- * the regional variations. Since some stations still transmit
- * only the legacy id and we don't ship regional variants of this
- * decoder as TV manufacturers do, this function can be used to
+ * the regional variants. Since some stations still transmit
+ * only the legacy three bit id and we don't ship regional variants
+ * of this decoder as TV manufacturers do, this function can be used to
  * set a default for the extended bits. The "factory default" is 16.
- **/
+ */
 void
 vbi_teletext_set_default_region(vbi_decoder *vbi, int default_region)
 {
@@ -2552,7 +2617,7 @@ vbi_teletext_set_default_region(vbi_decoder *vbi, int default_region)
 	vbi->vt.region = default_region;
 
 	for (i = 0; i < 9; i++) {
-		extension *ext = &vbi->vt.magazine[i].extension;
+		vt_extension *ext = &vbi->vt.magazine[i].extension;
 
 		ext->char_set[0] =
 		ext->char_set[1] =
@@ -2561,12 +2626,12 @@ vbi_teletext_set_default_region(vbi_decoder *vbi, int default_region)
 }
 
 /**
- * vbi_teletext_set_level:
- * @vbi: Initialized vbi decoding context.
- * @level: 
+ * @param vbi Initialized vbi decoding context.
+ * @param level 
  * 
- * Deprecated.
- **/
+ * @deprecated
+ * This became a parameter of vbi_fetch_vt_page().
+ */
 void
 vbi_teletext_set_level(vbi_decoder *vbi, int level)
 {
@@ -2579,13 +2644,13 @@ vbi_teletext_set_level(vbi_decoder *vbi, int level)
 }
 
 /**
- * vbi_teletext_desync:
- * @vbi: Initialized vbi decoding context.
+ * @internal
+ * @param vbi Initialized vbi decoding context.
  * 
  * This function must be called after desynchronisation
  * has been detected (i. e. vbi data has been lost)
  * to reset the Teletext decoder.
- **/
+ */
 void
 vbi_teletext_desync(vbi_decoder *vbi)
 {
@@ -2595,20 +2660,25 @@ vbi_teletext_desync(vbi_decoder *vbi)
 
 	for (i = 0; i < 8; i++)
 		vbi->vt.raw_page[i].page->function = PAGE_FUNCTION_DISCARD;
+
+	vbi_reset_page_clear(vbi->epg_pc + 0);
+	vbi_reset_page_clear(vbi->epg_pc + 1);
+
+	vbi->epg_pc[0].pfc.stream = 1;
+	vbi->epg_pc[1].pfc.stream = 2;
 }
 
 /**
- * vbi_teletext_channel_switched:
- * @vbi: Initialized vbi decoding context.
+ * @param vbi Initialized vbi decoding context.
  * 
  * This function must be called after a channel switch,
  * to reset the Teletext decoder.
- **/
+ */
 void
 vbi_teletext_channel_switched(vbi_decoder *vbi)
 {
-	magazine *mag;
-	extension *ext;
+	vt_magazine *mag;
+	vt_extension *ext;
 	int i, j;
 
 	vbi->vt.initial_page.pgno = 0x100;
@@ -2653,24 +2723,24 @@ vbi_teletext_channel_switched(vbi_decoder *vbi)
 
 
 /**
- * vbi_teletext_destroy:
- * @vbi: VBI decoding context.
+ * @internal
+ * @param vbi VBI decoding context.
  * 
- * This function is called during @vbi destruction
- * to destroy the Teletext subset of @vbi.
- **/
+ * This function is called during @a vbi destruction
+ * to destroy the Teletext subset of @a vbi object.
+ */
 void
 vbi_teletext_destroy(vbi_decoder *vbi)
 {
 }
 
 /**
- * vbi_teletext_init:
- * @vbi: VBI decoding context.
+ * @internal
+ * @param vbi VBI decoding context.
  * 
- * This function is called during @vbi initialization
- * to initialize the Teletext subset of @vbi.
- **/
+ * This function is called during @a vbi initialization
+ * to initialize the Teletext subset of @a vbi object.
+ */
 void
 vbi_teletext_init(vbi_decoder *vbi)
 {
@@ -2681,15 +2751,3 @@ vbi_teletext_init(vbi_decoder *vbi)
 
 	vbi_teletext_channel_switched(vbi);     /* Reset */
 }
-
-
-
-
-
-
-
-
-
-
-
-
