@@ -17,7 +17,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: teletext.c,v 1.2 2002/01/21 07:57:10 mschimek Exp $ */
+/* $Id: teletext.c,v 1.15 2004/04/04 21:46:38 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "../config.h"
@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h> /* strncasecmp */
 #include <ctype.h>
 #include <assert.h>
 
@@ -39,35 +40,47 @@
 #include "hamm.h"
 #include "lang.h"
 
+extern const char _zvbi_intl_domainname[];
+
 #ifndef _
-#ifdef ENABLE_NLS
+#  ifdef ENABLE_NLS
 #    include <libintl.h>
-#    define _(String) gettext (String)
+#    define _(String) dgettext (_zvbi_intl_domainname, String)
 #    ifdef gettext_noop
-#        define N_(String) gettext_noop (String)
+#      define N_(String) gettext_noop (String)
 #    else
-#        define N_(String) (String)
+#      define N_(String) (String)
 #    endif
-#else
-/* Stubs that do something close enough.  */
-#    define textdomain(String) (String)
-#    define gettext(String) (String)
-#    define dgettext(Domain,Message) (Message)
-#    define dcgettext(Domain,Message,Type) (Message)
-#    define bindtextdomain(Domain,Directory) (Domain)
+#  else /* Stubs that do something close enough.  */
+#    define gettext(Msgid) ((const char *) (Msgid))
+#    define dgettext(Domainname, Msgid) ((const char *) (Msgid))
+#    define dcgettext(Domainname, Msgid, Category) ((const char *) (Msgid))
+#    define ngettext(Msgid1, Msgid2, N) \
+       ((N) == 1 ? (const char *) (Msgid1) : (const char *) (Msgid2))
+#    define dngettext(Domainname, Msgid1, Msgid2, N) \
+       ((N) == 1 ? (const char *) (Msgid1) : (const char *) (Msgid2))
+#    define dcngettext(Domainname, Msgid1, Msgid2, N, Category) \
+       ((N) == 1 ? (const char *) (Msgid1) : (const char *) (Msgid2))
+#    define textdomain(Domainname) ((const char *) (Domainname))
+#    define bindtextdomain(Domainname, Dirname) ((const char *) (Dirname))
+#    define bind_textdomain_codeset(Domainname, Codeset) ((const char *) (Codeset))
 #    define _(String) (String)
 #    define N_(String) (String)
-#endif
+#  endif
 #endif
 
 #define DEBUG 0
 
-#if DEBUG
 #define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? \
                       '.' : ((c) & 0x7F))
+#if DEBUG
 #define printv(templ, args...) fprintf(stderr, templ ,##args)
 #else
-#define printv(templ, args...)
+#define printv(templ, args...)						\
+do {									\
+	if (0)								\
+		fprintf(stderr, templ ,##args);				\
+} while (0)
 #endif
 
 #define ROWS			25
@@ -163,7 +176,7 @@ flof_links(vbi_page *pg, vt_page *vtp)
  */
 
 static void character_set_designation(struct vbi_font_descr **font,
-				      extension *ext, vt_page *vtp);
+				      vt_extension *ext, vt_page *vtp);
 static void screen_color(vbi_page *pg, int flags, int color);
 
 static vbi_bool
@@ -238,11 +251,19 @@ top_label(vbi_decoder *vbi, vbi_page *pg, struct vbi_font_descr *font,
 	return FALSE;
 }
 
+static __inline__ vbi_pgno
+add_modulo			(vbi_pgno		pgno,
+				 int			incr)
+{
+	return ((pgno - 0x100 + incr) & 0x7FF) + 0x100;
+}
+
 static inline void
 top_navigation_bar(vbi_decoder *vbi, vbi_page *pg,
 		   vt_page *vtp)
 {
 	vbi_char ac;
+	vbi_pgno pgno1;
 	int i, got;
 
 	printv("PAGE MIP/BTT: %d\n", vbi->vt.page_info[vtp->pgno - 0x100].code);
@@ -260,14 +281,16 @@ top_navigation_bar(vbi_decoder *vbi, vbi_page *pg,
 	if (pg->page_opacity[1] != VBI_OPAQUE)
 		return;
 
-	for (i = vtp->pgno; i != vtp->pgno + 1; i = (i == 0) ? 0x89a : i - 1)
+	pgno1 = add_modulo (vtp->pgno, 1);
+
+	for (i = vtp->pgno; i != pgno1; i = add_modulo (i, -1))
 		if (vbi->vt.page_info[i - 0x100].code == VBI_TOP_BLOCK ||
 		    vbi->vt.page_info[i - 0x100].code == VBI_TOP_GROUP) {
 			top_label(vbi, pg, pg->font[0], 0, i, 32 + VBI_WHITE, 0);
 			break;
 		}
 
-	for (i = vtp->pgno + 1, got = FALSE; i != vtp->pgno; i = (i == 0x899) ? 0x100 : i + 1)
+	for (i = pgno1, got = FALSE; i != vtp->pgno; i = add_modulo (i, 1))
 		switch (vbi->vt.page_info[i - 0x100].code) {
 		case VBI_TOP_BLOCK:
 			top_label(vbi, pg, pg->font[0], 2, i, 32 + VBI_YELLOW, 2);
@@ -338,7 +361,7 @@ top_index(vbi_decoder *vbi, vbi_page *pg, int subno)
 	ait_entry *ait;
 	int i, j, k, n, lines;
 	int xpgno, xsubno;
-	extension *ext;
+	vt_extension *ext;
 	char *index_str;
 
 	pg->vbi = vbi;
@@ -380,7 +403,9 @@ top_index(vbi_decoder *vbi, vbi_page *pg, int subno)
 
 	ac.size = VBI_DOUBLE_SIZE;
 
-	/* NLS: Title of TOP Index page, Latin-1 only */
+	/* FIXME */
+	/* TRANSLATORS: Title of TOP Index page,
+	   for now please Latin-1 or ASCII only */
 	index_str = _("TOP Index");
 
 	for (i = 0; index_str[i]; i++) {
@@ -448,6 +473,266 @@ top_index(vbi_decoder *vbi, vbi_page *pg, int subno)
 	return 1;
 }
 
+struct pex26 {
+	signed			month		: 8;  /* 0 ... 11 */
+	signed			day		: 8;  /* 0 ... 30 */
+	signed			at1		: 16; /* min since 00:00 */
+	signed			at2		: 16; /* min since 00:00 */
+	signed			length		: 16; /* min */
+	unsigned		x26_cni		: 16; /* see tables.c */
+	unsigned		pty		: 8;
+	signed			lto		: 8;  /* +- 1/4 hr */
+	signed			row		: 8;  /* title 1 ... 23 */
+	signed			column		: 8;  /* title 0 ... 39 */
+	unsigned		caf		: 1;
+	unsigned				: 15;
+};
+
+static void
+dump_pex26(struct pex26 *pt, int n)
+{
+	int i;
+
+	for (i = 0; i < n; i++, pt++)
+		fprintf(stderr, "%2d: %02d-%02d %d:%02d (%d:%02d) +%d, "
+			"cni=%04x pty=%02x lto=%d tit=%d:%d caf=%d\n",
+			i, pt->month, pt->day,
+			pt->at1 / 60, pt->at1 % 60,
+			pt->at2 / 60, pt->at2 % 60,
+			pt->length,
+			pt->x26_cni, pt->pty, pt->lto,
+			pt->row, pt->column,
+			pt->caf);
+}
+
+#if 0
+
+/*
+
+type	pre	text		____      post       ____
+                               /                         \
+AT-1	+	zz.zz		+	%		<
+AT-1	+	zz.zz-zz.zz	+	%		<
+PTL	++	title		++	%%	::	<
+AT-1	%	zz.zz		+	%		<
+PW*)	%	hh
+LTO	%	0zz		+	%		<
+LTO	%	9zz		+	%		<
+AT-2	%	zzzz		+	%		<
+CNI*)	%	hhzzz		+	%		<
+AD*)	%	zzzzzz		+	%		<
+PTL	%%	title		++	%%	::	<
+AT-2	:	zzzz		+	%		<
+AD	:	zzzzzz		+	%		<
+PW	:%	hh		+	%		<
+PTY	:%	Fhh		+	%		<
+AT-2	:%	zzzz		+	%		<
+CNI	:%	hhzzz		+	%		<
+AD	:%	zzzzzz		+	%		<
+
++  colour code
+:  magenta
+%  conceal
+
+*) permitted when CNI, AD, PW combine
+
+Note ETS 300 231 Table 4 is wrong: '%' = 0x18; ',' = '+' | '%' | '<'
+
+*/
+
+/* to be rewritten */
+
+struct program_entry {
+	int			start;
+	int			stop;
+	int			at2;
+	int			ad;
+	int			cni;
+	int			pty;
+	int			lto;
+	uint16_t		title[200];
+};
+
+#define PMA_COLOUR /* 0x01, 0x02, 0x03, 0x04, 0x06, 0x07 */
+#define PMA_MAGENTA 0x05
+#define PMA_CONCEAL 0x18
+
+#define IS_PMA_CTRL(c) (((c) >= 0x01 && (c) <= 0x07) || (c) == PMA_CONCEAL)
+
+static int
+bcd2time(int bcd)
+{
+	int sec = bcd & 15;
+	int min = (bcd >> 8) & 15;
+
+	if (sec > 9 || min > 9 || (bcd & 0x00FF) > 0x0059)
+		return -1;
+//#warning hour check
+
+	return sec * 1 + min * 60
+		+ ((bcd >> 4) & 15) * 10
+		+ ((bcd >> 12) & 15) * 600;
+}
+
+static int
+pdc_method_a(vbi_page *pg, vt_page *vtp, struct program_entry *pe)
+{
+	int row, column;
+	int i;
+
+//	memset(pe, -1, sizeof(*pe));
+
+	i = 40;
+
+	for (row = 1; row <= 23; row++) {
+		for (column = 0; column <= 38;) {
+			int ctrl1 = vbi_parity(vtp->data.lop.raw[row][column]);
+			int ctrl2 = vbi_parity(vtp->data.lop.raw[row][column + 1]);
+
+fprintf(stderr, "%d %d %02x %02x\n", row, column, ctrl1, ctrl2);
+
+			if ((ctrl1 | ctrl2) < 0) {
+				return 0; /* hamming error */
+			} else if (!IS_PMA_CTRL(ctrl1)) {
+				column++;
+				continue;
+			}
+
+			if (ctrl1 == ctrl2 && ctrl1 != PMA_MAGENTA) {
+fprintf(stderr, "PTL %d %d\n", row, column);
+				/* title */
+				column += 2;fprintf(stderr, "%d %d %02x %02x\n", row, column, ctrl1, ctrl2);
+
+
+			} else {
+				/* numeral */
+				int digits, sep, value;
+fprintf(stderr, "NUM %d %d\n", row, column);
+				column += (ctrl1 == PMA_MAGENTA && ctrl2 == PMA_CONCEAL) ? 2 : 1;
+
+				sep = 0;
+				value = 0;
+
+				for (digits = 0; column < 40; column++) {
+					int c = vbi_parity(vtp->data.lop.raw[row][column]);
+
+					if (IS_PMA_CTRL(c)) {
+						break;
+					} else if (c >= 0x30 && c <= 0x39) {
+						value = value * 16 + c - 0x30;
+						digits++;
+					} else if (c >= 0x41 && c <= 0x46) {
+						if (digits >= 3)
+							goto invalid_pattern;
+						value = value * 16 + c + (0x0A - 0x41);
+						digits++;
+					} else if (c == 0x2E) {
+						if (digits != 2 && digits != 6)
+							goto invalid_pattern;
+						sep |= 1 << digits;
+					} else if (c == 0x2D) {
+						if (digits != 4)
+							goto invalid_pattern;
+						sep |= 1 << 4;
+					} else
+						goto invalid_pattern;
+				}
+
+				if (sep) {
+					if (ctrl1 == PMA_MAGENTA)
+						goto invalid_pattern;
+					if (ctrl1 == PMA_CONCEAL && digits != 4)
+						goto invalid_pattern;
+				}
+
+				switch (digits) {
+					int start, stop;
+
+				case 2:
+					/* Actually ctrl1 only permitted when combined */
+					if (ctrl1 != PMA_CONCEAL && ctrl2 != PMA_CONCEAL)
+						goto invalid_pattern;
+fprintf(stderr, "PW %02x\n", value);
+					/* PW */
+					break;
+
+				case 3:
+					if (ctrl1 == PMA_CONCEAL) {
+						if (value >= 0x100 && value < 0x900)
+							goto invalid_pattern;
+fprintf(stderr, "LTO %03x\n", value);
+						/* LTO */
+					} else if (ctrl2 == PMA_CONCEAL) {
+						if ((value -= 0xF00) < 0)
+						goto invalid_pattern;
+fprintf(stderr, "PTY %02x\n", value);
+						/* PTY */
+					} else
+						goto invalid_pattern;
+
+				case 4:
+					start = bcd2time(value);
+
+					if (start < 0)
+						goto invalid_pattern;
+
+					if (sep) {
+						if (ctrl1 == PMA_MAGENTA)
+							goto invalid_pattern;
+fprintf(stderr, "AT-1 %04x\n", value);
+						; /* AT-1 short */
+					} else if (ctrl1 == PMA_MAGENTA || ctrl1 == PMA_CONCEAL) {
+fprintf(stderr, "AT-2 %04x\n", value);
+						; /* AT-2 */
+					} else
+						goto invalid_pattern;
+
+					break;
+
+				case 5:
+					/* Actually ctrl1 only permitted when combined */
+					if ((ctrl1 != PMA_CONCEAL && ctrl2 != PMA_CONCEAL)
+					    || (value & 0x00F00) > 0x00900)
+						goto invalid_pattern;
+
+					/* CNI */
+fprintf(stderr, "CNI %05x\n", value);
+					break;
+
+				case 6:
+					/* Actually ctrl1 only permitted when combined */
+					if (ctrl1 != PMA_CONCEAL && ctrl2 != PMA_CONCEAL)
+						goto invalid_pattern;
+					/* AD */
+fprintf(stderr, "AD %06x\n", value);
+					break;
+
+				case 8:
+					start = bcd2time(value >> 16); 
+					stop = bcd2time(value);
+
+					if ((start | stop) < 0
+					    || ctrl1 == PMA_MAGENTA || ctrl1 == PMA_CONCEAL
+					    || sep != ((1 << 2) + (1 << 4) + (1 << 6)))
+						goto invalid_pattern;
+
+					/* AT-1 long */
+fprintf(stderr, "AT1 %08x\n", value);
+					break;
+
+				default:
+				invalid_pattern:
+					continue;
+				}
+			}
+		}
+	}
+
+	return 0; /* invalid */
+}
+
+#endif
+
 /*
  *  Zapzilla navigation
  */
@@ -606,17 +891,16 @@ zap_links(vbi_page *pg, int row)
 }
 
 /**
- * vbi_resolve_link:
- * @pg: With vbi_fetch_vt_page() obtained #vbi_page.
- * @column: Column 0 ... pg->columns - 1 of the character in question.
- * @row: Row 0 ... pg->rows - 1 of the character in question.
- * @ld: Place to store information about the link.
+ * @param pg With vbi_fetch_vt_page() obtained vbi_page.
+ * @param column Column 0 ... pg->columns - 1 of the character in question.
+ * @param row Row 0 ... pg->rows - 1 of the character in question.
+ * @param ld Place to store information about the link.
  * 
- * A #vbi_page (in practice only Teletext pages) may contain hyperlinks
+ * A vbi_page (in practice only Teletext pages) may contain hyperlinks
  * such as HTTP URLs, e-mail addresses or links to other pages. Characters
- * being part of a hyperlink have a set #vbi_char.link flag, this function
- * returns a more verbose description of the respective link.
- **/
+ * being part of a hyperlink have a set vbi_char->link flag, this function
+ * returns a more verbose description of the link.
+ */
 void
 vbi_resolve_link(vbi_page *pg, int column, int row, vbi_link *ld)
 {
@@ -678,14 +962,13 @@ vbi_resolve_link(vbi_page *pg, int column, int row, vbi_link *ld)
 }
 
 /**
- * vbi_resolve_home:
- * @pg: With vbi_fetch_vt_page() obtained #vbi_page.
- * @ld: Place to store information about the link.
+ * @param pg With vbi_fetch_vt_page() obtained vbi_page.
+ * @param ld Place to store information about the link.
  * 
  * All Teletext pages have a built-in home link, by default
- * page 100, but also the magazine intro page or another page
- * selected by the editor.
- **/
+ * page 100, but can also be the magazine intro page or another
+ * page selected by the editor.
+ */
 void
 vbi_resolve_home(vbi_page *pg, vbi_link *ld)
 {
@@ -722,20 +1005,19 @@ ait_title(vbi_decoder *vbi, vt_page *vtp, ait_entry *ait, char *buf)
 }
 
 /**
- * vbi_page_title:
- * @vbi: Initialized vbi decoding context.
- * @pgno: Page number, see #vbi_pgno.
- * @subno: Subpage number.
- * @buf: Place to store the titel, Latin-1 format, at least
+ * @param vbi Initialized vbi decoding context.
+ * @param pgno Page number, see vbi_pgno.
+ * @param subno Subpage number.
+ * @param buf Place to store the title, Latin-1 format, at least
  *   41 characters including the terminating zero.
  * 
  * Given a Teletext page number this function tries to deduce a
  * page title for bookmarks or other purposes, mainly from navigation
  * data. (XXX TODO: FLOF)
  * 
- * Return value: 
- * TRUE if a title has been found.
- **/
+ * @return
+ * @c TRUE if a title has been found.
+ */
 vbi_bool
 vbi_page_title(vbi_decoder *vbi, int pgno, int subno, char *buf)
 {
@@ -777,7 +1059,7 @@ vbi_page_title(vbi_decoder *vbi, int pgno, int subno, char *buf)
 
 static void
 character_set_designation(struct vbi_font_descr **font,
-			  extension *ext, vt_page *vtp)
+			  vt_extension *ext, vt_page *vtp)
 {
 	int i;
 
@@ -898,12 +1180,13 @@ resolve_obj_address(vbi_decoder *vbi, object_type type,
 /* FIXME: panels */
 
 static vbi_bool
-enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
+enhance(vbi_decoder *vbi, vt_magazine *mag, vt_extension *ext,
 	vbi_page *pg, vt_page *vtp,
 	object_type type, vt_triplet *p,
 	int max_triplets,
 	int inv_row, int inv_column,
-	vbi_wst_level max_level, vbi_bool header_only)
+	vbi_wst_level max_level, vbi_bool header_only,
+	struct pex26 *ptable)
 {
 	vbi_char ac, mac, *acp;
 	int active_column, active_row;
@@ -912,6 +1195,8 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 	struct vbi_font_descr *font;
 	int invert;
 	int drcs_s1[2];
+	struct pex26 *pt, ptmp;
+	int pdc_hr;
 
 	static void
 	flush(int column)
@@ -1120,7 +1405,21 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 
 	font = pg->font[0];
 
-	for (;max_triplets>0; p++, max_triplets--) {
+	if (ptable) {
+		ptmp.month	= -1;
+		ptmp.at1	= -1; /* n/a */
+		ptmp.length	= 0;
+		ptmp.x26_cni	= 0;
+		ptmp.pty	= 0;
+		ptmp.lto	= 0;
+
+		pt = ptable - 1;
+	} else
+		pt = &ptmp;
+
+	pdc_hr = 0;
+
+	for (; max_triplets>0; p++, max_triplets--) {
 		if (p->address >= COLUMNS) {
 			/*
 			 *  Row address triplets
@@ -1129,6 +1428,8 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 			int row = (p->address - COLUMNS) ? : (ROWS - 1);
 			int column = 0;
 
+			if (pdc_hr)
+				return FALSE; /* invalid */
 
 			switch (p->mode) {
 			case 0x00:		/* full screen color */
@@ -1200,7 +1501,59 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 
 			case 0x05:		/* reserved */
 			case 0x06:		/* reserved */
-			case 0x08 ... 0x0F:	/* PDC data */
+				break;
+
+			case 0x08:		/* PDC data - Country of Origin and Programme Source */
+				ptmp.x26_cni = p->address * 256 + p->data;
+				break;
+
+			case 0x09:		/* PDC data - Month and Day */
+				ptmp.month = (p->address & 15) - 1;
+				ptmp.day = (p->data >> 4) * 10 + (p->data & 15) - 1;
+				break;
+
+			case 0x0A:		/* PDC data - Cursor Row and Announced Starting Time Hours */
+				if (!ptable) {
+					break;
+				} else if ((ptmp.month | ptmp.x26_cni) < 0) {
+					return FALSE;
+				} else if ((ptable - pt) > 22) {
+					return FALSE;
+				}
+
+				*++pt = ptmp;
+
+				/* fall through */
+
+			case 0x0B:		/* PDC data - Cursor Row and Announced Finishing Time Hours */
+				s = (p->data & 15) * 60;
+
+				if (p->mode == 0x0A) {
+					pt->at2 = ((p->data & 0x30) >> 4) * 600 + s;
+					pt->length = 0;
+					pt->row = row;
+					pt->caf = !!(p->data & 0x40);
+				} else {
+					pt->length = ((p->data & 0x70) >> 4) * 600 + s;
+				}
+
+				pdc_hr = p->mode;
+
+				break;
+
+			case 0x0C:		/* PDC data - Cursor Row and Local Time Offset */
+				ptmp.lto = (p->data & 0x40) ? ((~0x7F) | p->data) : p->data;
+				break;
+
+			case 0x0D:		/* PDC data - Series Identifier and Series Code */
+				if (p->address == 0x30) {
+					break;
+				}
+				pt->pty = 0x80 + p->data;
+				break;
+
+			case 0x0E:		/* reserved */
+			case 0x0F:		/* reserved */
 				break;
 
 			case 0x10:		/* origin modifier */
@@ -1307,7 +1660,7 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 				if (!enhance(vbi, mag, ext, pg, vtp, new_type, trip,
 					     remaining_max_triplets,
 					     row + offset_row, column + offset_column,
-					     max_level, header_only))
+					     max_level, header_only, NULL))
 					return FALSE;
 
 				printv("... object done\n");
@@ -1413,8 +1766,48 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 
 			case 0x04:		/* reserved */
 			case 0x05:		/* reserved */
+				break;
+
+			case 0x06:		/* PDC data - Cursor Column and Announced Starting */
+						/* and Finishing Time Minutes */
+				if (!ptable)
+					break;
+
+				s = (p->data >> 4) * 10 + (p->data & 15);
+
+				if (pdc_hr == 0x0A) {
+					pt->at2 += s;
+
+					if (pt > ptable && pt[-1].length == 0) {
+						pt[-1].length = pt->at2 - pt[-1].at2;
+
+						if (pt->at2 < pt[-1].at2)
+							pt[-1].length += 24 * 60;
+
+						if (pt[-1].length >= 12 * 60) {
+							/* bullshit */
+							pt[-1] = pt[0];
+							pt--;
+						}
+					}
+				} else if (pdc_hr == 0x0B) {
+					pt->length += s;
+
+					if (pt->length >= 4 * 600) {
+						pt->length -= 4 * 600;
+					} else {
+						if (pt->length < pt->at2)
+							pt->length += 24 * 60;
+
+						pt->length -= pt->at2;
+					}
+				} else {
+					return FALSE;
+				}
+
+				pt->column = column;
+				pdc_hr = 0;
 
-			case 0x06:		/* PDC data */
 				break;
 
 			case 0x07:		/* additional flash functions */
@@ -1641,9 +2034,9 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 					unicode = vbi_teletext_composed_unicode(
 						p->mode - 0x10, p->data);
 			store:
-					printv("enh row %d col %d print 0x%02x/0x%02x -> 0x%04x %c\n",
+					printv("enh row %d col %d print 0x%02x/0x%02x -> 0x%04x\n",
 						active_row, active_column, p->mode, p->data,
-						gl, unicode & 0xFF);
+						unicode);
 
 					ac.unicode = unicode;
 					mac.unicode = ~0;
@@ -1656,21 +2049,29 @@ enhance(vbi_decoder *vbi, magazine *mag,	extension *ext,
 
 swedish:
 
-#if 0
-	acp = pg->data[0];
+	if (ptable) {
+		if (pt >= ptable && (pdc_hr || pt->length == 0))
+			pt--; /* incomplete start or end tag */
 
-	for (active_row = 0; active_row < ROWS; active_row++) {
-		printv("%2d: ", active_row);
+		if (1)
+			dump_pex26(ptable, pt - ptable + 1);
+	}
 
-		for (active_column = 0; active_column < COLUMNS; acp++, active_column++) {
-			printv("%04x ", acp->unicode);
-		}
+	if (0) {
+		acp = pg->text;
+
+		for (active_row = 0; active_row < ROWS; active_row++) {
+			printv("%2d: ", active_row);
+
+			for (active_column = 0; active_column < COLUMNS; acp++, active_column++) {
+				printv("%04x ", acp->unicode);
+			}
 
-		printv("\n");
+			printv("\n");
 
-		acp += EXT_COLUMNS - COLUMNS;
+			acp += EXT_COLUMNS - COLUMNS;
+		}
 	}
-#endif
 
 	return TRUE;
 }
@@ -1760,11 +2161,11 @@ post_enhance(vbi_page *pg, int display_rows)
 }
 
 static inline vbi_bool
-default_object_invocation(vbi_decoder *vbi, magazine *mag,
-	extension *ext, vbi_page *pg, vt_page *vtp,
+default_object_invocation(vbi_decoder *vbi, vt_magazine *mag,
+	vt_extension *ext, vbi_page *pg, vt_page *vtp,
 	vbi_wst_level max_level, vbi_bool header_only)
 {
-	pop_link *pop;
+	vt_pop_link *pop;
 	int i, order;
 
 	if (!(i = mag->pop_lut[vtp->pgno & 0xFF]))
@@ -1802,7 +2203,7 @@ default_object_invocation(vbi_decoder *vbi, magazine *mag,
 
 		if (!enhance(vbi, mag, ext, pg, vtp, type, trip,
 			     remaining_max_triplets, 0, 0, max_level,
-			     header_only))
+			     header_only, NULL))
 			return FALSE;
 	}
 
@@ -1810,21 +2211,21 @@ default_object_invocation(vbi_decoder *vbi, magazine *mag,
 }
 
 /**
- * vbi_format_vt_page:
- * @vbi: Initialized vbi decoding context.
- * @pg: Place to store the formatted page.
- * @vtp: Raw Teletext page. 
- * @max_level: Format the page at this Teletext implementation level.
- * @display_rows: Number of rows to format, between 1 ... 25.
- * @navigation: Analyse the page and add navigation links,
+ * @internal
+ * @param vbi Initialized vbi_decoder context.
+ * @param pg Place to store the formatted page.
+ * @param vtp Raw Teletext page. 
+ * @param max_level Format the page at this Teletext implementation level.
+ * @param display_rows Number of rows to format, between 1 ... 25.
+ * @param navigation Analyse the page and add navigation links,
  *   including TOP and FLOF.
  * 
- * Format a page @pg from a raw Teletext page @vtp. This function is
+ * Format a page @a pg from a raw Teletext page @a vtp. This function is
  * used internally by libzvbi only.
  * 
- * Return value: 
- * TRUE if the page could be formatted.
- **/
+ * @return
+ * @c TRUE if the page could be formatted.
+ */
 int
 vbi_format_vt_page(vbi_decoder *vbi,
 		   vbi_page *pg, vt_page *vtp,
@@ -1832,8 +2233,8 @@ vbi_format_vt_page(vbi_decoder *vbi,
 		   int display_rows, vbi_bool navigation)
 {
 	char buf[16];
-	magazine *mag;
-	extension *ext;
+	vt_magazine *mag;
+	vt_extension *ext;
 	int column, row, i;
 
 	if (vtp->function != PAGE_FUNCTION_LOP &&
@@ -2096,21 +2497,23 @@ vbi_format_vt_page(vbi_decoder *vbi,
 			pg->double_height_lower |= 1 << row;
 		}
 	}
-#if 0
-	if (row < ROWS) {
-		vbi_char ac;
 
-		memset(&ac, 0, sizeof(ac));
+	if (0) {
+		if (row < ROWS) {
+			vbi_char ac;
 
-		ac.foreground	= ext->foreground_clut + WHITE;
-		ac.background	= ext->background_clut + BLACK;
-		ac.opacity	= pg->page_opacity[1];
-		ac.unicode	= 0x0020;
+			memset(&ac, 0, sizeof(ac));
 
-		for (i = row * EXT_COLUMNS; i < ROWS * EXT_COLUMNS; i++)
-			pg->text[i] = ac;
+			ac.foreground	= ext->foreground_clut + VBI_WHITE;
+			ac.background	= ext->background_clut + VBI_BLACK;
+			ac.opacity	= pg->page_opacity[1];
+			ac.unicode	= 0x0020;
+
+			for (i = row * EXT_COLUMNS; i < ROWS * EXT_COLUMNS; i++)
+				pg->text[i] = ac;
+		}
 	}
-#endif
+
 	/* Local enhancement data and objects */
 
 	if (max_level >= VBI_WST_LEVEL_1p5 && display_rows > 0) {
@@ -2128,7 +2531,8 @@ vbi_format_vt_page(vbi_decoder *vbi,
 			printv("enhancement packets %08x\n", vtp->enh_lines);
 			success = enhance(vbi, mag, ext, pg, vtp, LOCAL_ENHANCEMENT_DATA,
 				vtp->data.enh_lop.enh, elements(vtp->data.enh_lop.enh),
-				0, 0, max_level, display_rows == 1);
+				0, 0, max_level, display_rows == 1, NULL);
+							   // here --^
 		} else
 			success = default_object_invocation(vbi, mag, ext, pg, vtp,
 							    max_level, display_rows == 1);
@@ -2164,6 +2568,24 @@ vbi_format_vt_page(vbi_decoder *vbi,
 					flof_navigation_bar(pg, vtp);
 			} else if (vbi->vt.top)
 				top_navigation_bar(vbi, pg, vtp);
+
+//			pdc_method_a(pg, vtp, NULL);
+		}
+	}
+
+	if (0) {
+		vbi_char *acp;
+
+		for (row = ROWS - 1, acp = pg->text + EXT_COLUMNS * row; row < ROWS; row++) {
+			fprintf(stderr, "%2d: ", row);
+
+			for (column = 0; column < COLUMNS; acp++, column++) {
+				fprintf(stderr, "%04x ", acp->unicode);
+			}
+
+			fprintf(stderr, "\n");
+
+			acp += EXT_COLUMNS - COLUMNS;
 		}
 	}
 
@@ -2171,33 +2593,32 @@ vbi_format_vt_page(vbi_decoder *vbi,
 }
 
 /**
- * vbi_fetch_vt_page:
- * @vbi: Initialized VBI decoding context.
- * @pg: Place to store the formatted page.
- * @pgno: Page number of the page to fetch, see #vbi_pgno.
- * @subno: Subpage number to fetch (optional #VBI_ANY_SUBNO).
- * @max_level: Format the page at this Teletext implementation level.
- * @display_rows: Number of rows to format, between 1 ... 25.
- * @navigation: Analyse the page and add navigation links,
+ * @param vbi Initialized vbi_decoder context.
+ * @param pg Place to store the formatted page.
+ * @param pgno Page number of the page to fetch, see vbi_pgno.
+ * @param subno Subpage number to fetch (optional @c VBI_ANY_SUBNO).
+ * @param max_level Format the page at this Teletext implementation level.
+ * @param display_rows Number of rows to format, between 1 ... 25.
+ * @param navigation Analyse the page and add navigation links,
  *   including TOP and FLOF.
  * 
- * Fetches a Teletext page designated by @pgno and @subno from the
- * cache, formats and stores it in @pg. Formatting is limited to row
- * 0 ... @display_rows - 1 inclusive. The really useful values
+ * Fetches a Teletext page designated by @a pgno and @a subno from the
+ * cache, formats and stores it in @a pg. Formatting is limited to row
+ * 0 ... @a display_rows - 1 inclusive. The really useful values
  * are 1 (format header only) or 25 (everything). Likewise
- * @navigation can be used to save unnecessary formatting time.
+ * @a navigation can be used to save unnecessary formatting time.
  * 
  * Although safe to do, this function is not supposed to be called from
  * an event handler since rendering may block decoding for extended
  * periods of time.
  *
- * Return value:
- * FALSE if the page is not cached or could not be formatted
- * for other reasons, for instance a data page not intended for
+ * @return
+ * @c FALSE if the page is not cached or could not be formatted
+ * for other reasons, for instance is a data page not intended for
  * display. Level 2.5/3.5 pages which could not be formatted e. g.
  * due to referencing data pages not in cache are formatted at a
  * lower level.
- **/
+ */
 vbi_bool
 vbi_fetch_vt_page(vbi_decoder *vbi, vbi_page *pg,
 		  vbi_pgno pgno, vbi_subno subno,
